@@ -35,11 +35,11 @@ import Data.SBV.Core.Model     (Metric(..), SymTuple)
 import Data.SBV.Core.Symbolic  (Objective, OptimizeStyle, Result, VarContext,
                                 Symbolic, SBVRunMode, SMTConfig, SVal)
 import Data.SBV.Control.Types  (SMTOption)
-import Data.SBV.Provers.Prover (Provable, SExecutable, ThmResult)
+import Data.SBV.Provers.Prover (Provable, Satisfiable, SExecutable, ThmResult)
 import Data.SBV.SMT.SMT        (AllSatResult, SafeResult, SatResult,
                                 OptimizeResult)
 
-import GHC.TypeLits
+import GHC.TypeLits (KnownNat, TypeError, ErrorMessage(..))
 import Data.Kind
 
 import Data.Int
@@ -51,48 +51,6 @@ import qualified Data.SBV.Core.Model     as Trans
 import qualified Data.SBV.Core.Symbolic  as Trans
 import qualified Data.SBV.Provers.Prover as Trans
 
--- Data.SBV.Provers.Prover:
-
--- | Turns a value into a universally quantified predicate, internally naming the inputs.
--- In this case the sbv library will use names of the form @s1, s2@, etc. to name these variables
--- Example:
---
--- >  forAll_ $ \(x::SWord8) y -> x `shiftL` 2 .== y
---
--- is a predicate with two arguments, captured using an ordinary Haskell function. Internally,
--- @x@ will be named @s0@ and @y@ will be named @s1@.
---
--- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.forAll_'
-forAll_ :: Provable a => a -> Symbolic SBool
-forAll_ = Trans.forAll_
-
--- | Turns a value into a predicate, allowing users to provide names for the inputs.
--- If the user does not provide enough number of names for the variables, the remaining ones
--- will be internally generated. Note that the names are only used for printing models and has no
--- other significance; in particular, we do not check that they are unique. Example:
---
--- >  forAll ["x", "y"] $ \(x::SWord8) y -> x `shiftL` 2 .== y
---
--- This is the same as above, except the variables will be named @x@ and @y@ respectively,
--- simplifying the counter-examples when they are printed.
---
--- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.forAll'
-forAll :: Provable a => [String] -> a -> Symbolic SBool
-forAll = Trans.forAll
-
--- | Turns a value into an existentially quantified predicate. (Indeed, 'exists' would have been
--- a better choice here for the name, but alas it's already taken.)
---
--- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.forSome_'
-forSome_ :: Provable a => a -> Symbolic SBool
-forSome_ = Trans.forSome_
-
--- | Version of 'forSome' that allows user defined names.
---
--- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.forSome'
-forSome :: Provable a => [String] -> a -> Symbolic SBool
-forSome = Trans.forSome
-
 -- | Prove a predicate, using the default solver.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.prove'
@@ -120,32 +78,32 @@ dproveWith = Trans.dproveWith
 -- | Find a satisfying assignment for a predicate, using the default solver.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sat'
-sat :: Provable a => a -> IO SatResult
+sat :: Satisfiable a => a -> IO SatResult
 sat = Trans.sat
 
 -- | Find a satisfying assignment using the given SMT-solver.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.satWith'
-satWith :: Provable a => SMTConfig -> a -> IO SatResult
+satWith :: Satisfiable a => SMTConfig -> a -> IO SatResult
 satWith = Trans.satWith
 
 -- | Find a delta-satisfying assignment for a predicate, using the default solver for delta-satisfiability.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.dsat'
-dsat :: Provable a => a -> IO SatResult
+dsat :: Satisfiable a => a -> IO SatResult
 dsat = Trans.dsat
 
 -- | Find a satisfying assignment using the given SMT-solver.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.satWith'
-dsatWith :: Provable a => SMTConfig -> a -> IO SatResult
+dsatWith :: Satisfiable a => SMTConfig -> a -> IO SatResult
 dsatWith = Trans.dsatWith
 
 -- | Find all satisfying assignments, using the default solver.
 -- Equivalent to @'allSatWith' 'Data.SBV.defaultSMTCfg'@. See 'allSatWith' for details.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.allSat'
-allSat :: Provable a => a -> IO AllSatResult
+allSat :: Satisfiable a => a -> IO AllSatResult
 allSat = Trans.allSat
 
 -- | Return all satisfying assignments for a predicate.
@@ -161,32 +119,32 @@ allSat = Trans.allSat
 --  Find all satisfying assignments using the given SMT-solver
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.allSatWith'
-allSatWith :: Provable a => SMTConfig -> a -> IO AllSatResult
+allSatWith :: Satisfiable a => SMTConfig -> a -> IO AllSatResult
 allSatWith = Trans.allSatWith
 
 -- | Optimize a given collection of `Objective`s.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.optimize'
-optimize :: Provable a => OptimizeStyle -> a -> IO OptimizeResult
+optimize :: Satisfiable a => OptimizeStyle -> a -> IO OptimizeResult
 optimize = Trans.optimize
 
 -- | Optimizes the objectives using the given SMT-solver.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.optimizeWith'
-optimizeWith :: Provable a => SMTConfig -> OptimizeStyle -> a -> IO OptimizeResult
+optimizeWith :: Satisfiable a => SMTConfig -> OptimizeStyle -> a -> IO OptimizeResult
 optimizeWith = Trans.optimizeWith
 
--- | Check if the constraints given are consistent, using the default solver.
+-- | Check if the constraints given are consistent in a prove call using the default solver.
 --
--- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.isVacuous'
-isVacuous :: Provable a => a -> IO Bool
-isVacuous = Trans.isVacuous
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.isVacuousProof'
+isVacuousProof :: Provable a => a -> IO Bool
+isVacuousProof = Trans.isVacuousProof
 
--- | Determine if the constraints are vacuous using the given SMT-solver.
+-- | Determine if the constraints are vacuous in a SAT call using the given SMT-solver.
 --
--- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.isVacuousWith'
-isVacuousWith :: Provable a => SMTConfig -> a -> IO Bool
-isVacuousWith = Trans.isVacuousWith
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.isVacuousProofWith'
+isVacuousProofWith :: Provable a => SMTConfig -> a -> IO Bool
+isVacuousProofWith = Trans.isVacuousProofWith
 
 -- | Checks theoremhood using the default solver.
 --
@@ -203,13 +161,13 @@ isTheoremWith = Trans.isTheoremWith
 -- | Checks satisfiability using the default solver.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.isSatisfiable'
-isSatisfiable :: Provable a => a -> IO Bool
+isSatisfiable :: Satisfiable a => a -> IO Bool
 isSatisfiable = Trans.isSatisfiable
 
 -- | Check whether a given property is satisfiable.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.isSatisfiableWith'
-isSatisfiableWith :: Provable a => SMTConfig -> a -> IO Bool
+isSatisfiableWith :: Satisfiable a => SMTConfig -> a -> IO Bool
 isSatisfiableWith = Trans.isSatisfiableWith
 
 -- | Run an arbitrary symbolic computation, equivalent to @'runSMTWith' 'Data.SBV.defaultSMTCfg'@
@@ -224,12 +182,10 @@ runSMT = Trans.runSMT
 runSMTWith :: SMTConfig -> Symbolic a -> IO a
 runSMTWith = Trans.runSMTWith
 
--- | NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sName_'
-sName_ :: SExecutable IO a => a -> Symbolic ()
-sName_ = Trans.sName_
-
--- | NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sName'
-sName :: SExecutable IO a => [String] -> a -> Symbolic ()
+-- | Create an argument for a name used in a safety-checking call.
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sName_'
+sName :: SExecutable IO a => a -> Symbolic ()
 sName = Trans.sName
 
 -- | Check safety using the default solver.
@@ -265,42 +221,6 @@ sbvToSymSV = Trans.sbvToSymSV
 output :: Outputtable a => a -> Symbolic a
 output = Trans.output
 
--- | Create a user named input (universal)
---
--- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.forall'
-forall :: SymVal a => String -> Symbolic (SBV a)
-forall = Trans.forall
-
--- | Create an automatically named input
---
--- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.forall_'
-forall_ :: SymVal a => Symbolic (SBV a)
-forall_ = Trans.forall_
-
--- | Get a bunch of new words
---
--- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.mkForallVars'
-mkForallVars :: SymVal a => Int -> Symbolic [SBV a]
-mkForallVars = Trans.mkForallVars
-
--- | Create an existential variable
---
--- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.exists'
-exists :: SymVal a => String -> Symbolic (SBV a)
-exists = Trans.exists
-
--- | Create an automatically named existential variable
---
--- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.exists_'
-exists_ :: SymVal a => Symbolic (SBV a)
-exists_ = Trans.exists_
-
--- | Create a bunch of existentials
---
--- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.mkExistVars'
-mkExistVars :: SymVal a => Int -> Symbolic [SBV a]
-mkExistVars = Trans.mkExistVars
-
 -- | Create a free variable, universal in a proof, existential in sat
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.free'
@@ -417,7 +337,7 @@ sWord16s = Trans.sWord16s
 sWord32 :: String -> Symbolic SWord32
 sWord32 = Trans.sWord32
 
--- | Declare an unamed 'SWord32'
+-- | Declare an unnamed 'SWord32'
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sWord32_'
 sWord32_ :: Symbolic SWord32
@@ -920,7 +840,7 @@ svToSymSV = Trans.svToSymSV
 -- | Run a symbolic computation, and return a extra value paired up with the 'Result'
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.runSymbolic'
-runSymbolic :: SBVRunMode -> Symbolic a -> IO (a, Result)
+runSymbolic :: SMTConfig -> SBVRunMode -> Symbolic a -> IO (a, Result)
 runSymbolic = Trans.runSymbolic
 
 -- | Add a new option
@@ -978,6 +898,7 @@ type family FromSized (t :: Type) :: Type where
    FromSized (SInt  32) = SInt32
    FromSized (SInt  64) = SInt64
 
+-- | Capture the correspondence, in terms of a constraint
 type family FromSizedCstr (t :: Type) :: Constraint where
    FromSizedCstr (WordN  8) = ()
    FromSizedCstr (WordN 16) = ()
@@ -1043,6 +964,7 @@ type family ToSized (t :: Type) :: Type where
    ToSized SInt32  = SInt  32
    ToSized SInt64  = SInt  64
 
+-- | Capture the correspondence in terms of a constraint
 type family ToSizedCstr (t :: Type) :: Constraint where
    ToSizedCstr Word8   = ()
    ToSizedCstr Word16  = ()
