@@ -11,8 +11,9 @@
 
 {-# LANGUAGE CPP           #-}
 {-# LANGUAGE PatternGuards #-}
+{-# LANGUAGE TupleSections #-}
 
-{-# OPTIONS_GHC -Wall -Werror #-}
+{-# OPTIONS_GHC -Wall -Werror -Wno-incomplete-uni-patterns #-}
 
 module Data.SBV.Compilers.C(compileToC, compileToCLib, compileToC', compileToCLib') where
 
@@ -25,6 +26,8 @@ import qualified Data.Set      as Set (member, union, unions, empty, toList, sin
 import System.FilePath                (takeBaseName, replaceExtension)
 import System.Random
 
+import Data.SBV.Core.Symbolic (ResultInp(..), ProgInfo(..))
+
 -- Work around the fact that GHC 8.4.1 started exporting <>.. Hmm..
 import Text.PrettyPrint.HughesPJ
 import qualified Text.PrettyPrint.HughesPJ as P ((<>))
@@ -444,7 +447,7 @@ genDriver cfg randVals fn inps outs mbRet = [pre, header, body, post]
 
 -- | Generate the C program
 genCProg :: CgConfig -> String -> Doc -> Result -> [(String, CgVal)] -> [(String, CgVal)] -> Maybe SV -> Doc -> ([Doc], [String])
-genCProg cfg fn proto (Result kindInfo _tvals _ovals cgs ins (_, preConsts) tbls arrs _uis _axioms (SBVPgm asgns) cstrs origAsserts _) inVars outVars mbRet extDecls
+genCProg cfg fn proto (Result pinfo kindInfo _tvals _ovals cgs topInps (_, preConsts) tbls arrs _uis axioms (SBVPgm asgns) cstrs origAsserts _) inVars outVars mbRet extDecls
   | isNothing (cgInteger cfg) && KUnbounded `Set.member` kindInfo
   = error $ "SBV->C: Unbounded integers are not supported by the C compiler."
           ++ "\nUse 'cgIntegerSize' to specify a fixed size for SInteger representation."
@@ -467,12 +470,16 @@ genCProg cfg fn proto (Result kindInfo _tvals _ovals cgs ins (_, preConsts) tbls
           ++ "\nUse 'cgSRealType' to specify a custom type for SReal representation."
   | not (null usorts)
   = error $ "SBV->C: Cannot compile functions with uninterpreted sorts: " ++ intercalate ", " usorts
+  | hasQuants pinfo
+  = error "SBV->C: Cannot compile in the presence of quantified variables."
+  | not $ null (progSpecialRels pinfo)
+  = error "SBV->C: Cannot compile in the presence of special relations."
+  | not (null axioms)
+  = error "SBV->C: Cannot compile in the presence of 'smtFunction' definitions, use 'compileToCLib' instead."
   | not (null cstrs)
   = tbd "Explicit constraints"
   | not (null arrs)
   = tbd "User specified arrays"
-  | needsExistentials (map fst (fst ins))
-  = error "SBV->C: Cannot compile functions with existentially quantified variables."
   | True
   = ([pre, header, post], flagsNeeded)
  where asserts | cgIgnoreAsserts cfg = []
@@ -494,7 +501,7 @@ genCProg cfg fn proto (Result kindInfo _tvals _ovals cgs ins (_, preConsts) tbls
              $$ nest 2 (   vcat (concatMap (genIO True . (\v -> (isAlive v, v))) inVars)
                         $$ vcat (merge (map genTbl tbls) (map genAsgn assignments) (map genAssert asserts))
                         $$ sepIf (not (null assignments) || not (null tbls))
-                        $$ vcat (concatMap (genIO False) (zip (repeat True) outVars))
+                        $$ vcat (concatMap (genIO False . (True,)) outVars)
                         $$ maybe empty mkRet mbRet
                        )
              $$ text "}"
@@ -512,7 +519,12 @@ genCProg cfg fn proto (Result kindInfo _tvals _ovals cgs ins (_, preConsts) tbls
                          $$ vcat (map text ls)
                          $$ text ""
 
-       typeWidth = getMax 0 $ [len (kindOf s) | (s, _) <- assignments] ++ [len (kindOf s) | (_, NamedSymVar s _) <- fst ins]
+       ins = case topInps of
+               ResultTopInps (is, []) -> is
+               ResultTopInps is       -> die $ "Unexpected trackers: " ++ show is
+               ResultLamInps is       -> die $ "Unexpected inputs  : " ++ show is
+
+       typeWidth = getMax 0 $ [len (kindOf s) | (s, _) <- assignments] ++ [len (kindOf s) | NamedSymVar s _ <- ins]
                 where len KReal{}            = 5
                       len KFloat{}           = 6 -- SFloat
                       len KDouble{}          = 7 -- SDouble
@@ -574,8 +586,8 @@ genCProg cfg fn proto (Result kindInfo _tvals _ovals cgs ins (_, preConsts) tbls
          where static   = if location == -1 then text "static" else empty
                location = maximum (-1 : map getNodeId elts)
 
-       getNodeId s@(SV _ (NodeId n)) | isConst s = -1
-                                     | True      = n
+       getNodeId s@(SV _ (NodeId (_, n))) | isConst s = -1
+                                          | True      = n
 
        genAsgn :: (SV, SBVExpr) -> (Int, Doc)
        genAsgn (sv, n) = (getNodeId sv, ppExpr cfg consts n (declSV typeWidth sv) (declSVNoConst typeWidth sv) P.<> semi)
@@ -823,6 +835,9 @@ ppExpr cfg consts (SBVApp op opArgs) lhs (typ, var)
         p o [a, b]
           | Just co <- lookup o cBinOps
           = a <+> text co <+> b
+
+        p Implies [a, b] | kindOf (head opArgs) == KBool = parens (text "!" P.<> a <+> text "||" <+> b)
+
         p NotEqual xs = mkDistinct xs
         p o args = die $ "Received operator " ++ show o ++ " applied to " ++ show args
 
@@ -1054,4 +1069,4 @@ getLDFlag (o, k) = flag o
                        , FP_IsZero
                        ]
 
-{-# ANN module ("HLint: ignore Redundant lambda" :: String) #-}
+{- HLint ignore module "Redundant lambda" -}
