@@ -57,6 +57,8 @@ import qualified Text.PrettyPrint.HughesPJ as P (render)
 import Data.SBV.Core.Data
 import Data.SBV.Core.Symbolic (MonadSymbolic(..), svToSymSV, svMkSymVar, outputSVal, VarContext(..))
 
+import Data.SBV.Provers.Prover(defaultSMTCfg)
+
 #if MIN_VERSION_base(4,11,0)
 import Control.Monad.Fail as Fail
 #endif
@@ -262,7 +264,7 @@ svCgReturnArr vs
 
 -- | Creates an atomic input in the generated code.
 cgInput :: SymVal a => String -> SBVCodeGen (SBV a)
-cgInput nm = do r <- forall_
+cgInput nm = do r  <- free_
                 sv <- sbvToSymSV r
                 modify' (\s -> s { cgInputs = (nm, CgAtomic sv) : cgInputs s })
                 return r
@@ -271,7 +273,7 @@ cgInput nm = do r <- forall_
 cgInputArr :: SymVal a => Int -> String -> SBVCodeGen [SBV a]
 cgInputArr sz nm
   | sz < 1 = error $ "SBV.cgInputArr: Array inputs must have at least one element, given " ++ show sz ++ " for " ++ show nm
-  | True   = do rs <- mapM (const forall_) [1..sz]
+  | True   = do rs <- mapM (const free_) [1..sz]
                 sws <- mapM sbvToSymSV rs
                 modify' (\s -> s { cgInputs = (nm, CgArray sws) : cgInputs s })
                 return rs
@@ -337,9 +339,9 @@ instance Show CgPgmBundle where
 -- of makefiles, source code, headers, etc.
 codeGen :: CgTarget l => l -> CgConfig -> String -> SBVCodeGen a -> IO (a, CgConfig, CgPgmBundle)
 codeGen l cgConfig nm (SBVCodeGen comp) = do
-   ((retVal, st'), res) <- runSymbolic CodeGen $ runStateT comp initCgState { cgFinalConfig = cgConfig }
-   let st = st' { cgInputs       = reverse (cgInputs st')
-                , cgOutputs      = reverse (cgOutputs st')
+   ((retVal, st'), res) <- runSymbolic defaultSMTCfg CodeGen $ runStateT comp initCgState { cgFinalConfig = cgConfig }
+   let st = st' { cgInputs  = reverse (cgInputs st')
+                , cgOutputs = reverse (cgOutputs st')
                 }
        allNamedVars = map fst (cgInputs st ++ cgOutputs st)
        dupNames = allNamedVars \\ nub allNamedVars
