@@ -11,6 +11,8 @@
 -- @Data.SBV.Control@, where we restrict the underlying monad to be IO.
 -----------------------------------------------------------------------------
 
+{-# LANGUAGE FlexibleContexts #-}
+
 {-# OPTIONS_GHC -Wall -Werror #-}
 
 module Data.SBV.Control.BaseIO where
@@ -18,8 +20,8 @@ module Data.SBV.Control.BaseIO where
 import Data.SBV.Control.Query (Assignment)
 import Data.SBV.Control.Types (CheckSatResult, SMTInfoFlag, SMTInfoResponse, SMTOption, SMTReasonUnknown)
 import Data.SBV.Core.Concrete (CV)
-import Data.SBV.Core.Data     (HasKind, Symbolic, SymArray, SymVal, SBool, SBV, SBVType)
-import Data.SBV.Core.Symbolic (Query, QueryContext, QueryState, State, SMTModel, SMTResult, SV, Name)
+import Data.SBV.Core.Data     (HasKind, Symbolic, SymArray, SymVal, SBool, SBV, SBVType, Lambda(..))
+import Data.SBV.Core.Symbolic (Query, QueryContext, QueryState, State, SMTModel, SMTResult, SV, Name, SymbolicT)
 
 import qualified Data.SBV.Control.Query as Trans
 import qualified Data.SBV.Control.Utils as Trans
@@ -57,7 +59,7 @@ getObservables = Trans.getObservables
 -- | Get the uninterpreted constants/functions recorded during a run.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.Control.getUIs'
-getUIs :: Query [(String, SBVType)]
+getUIs :: Query [(String, (Maybe [String], SBVType))]
 getUIs = Trans.getUIs
 
 -- | Issue check-sat and get an SMT Result out.
@@ -299,6 +301,27 @@ getInterpolantMathSAT = Trans.getInterpolantMathSAT
 getInterpolantZ3 :: [SBool] -> Query String
 getInterpolantZ3 = Trans.getInterpolantZ3
 
+-- | Get an abduct. The first argument is a conjecture. The return value will be an assertion
+-- such that in addition with the existing assertions you have, will imply this conjecture.
+-- The second argument is the grammar which guides the synthesis of this abduct, if given.
+-- Note that SBV doesn't do any checking on the grammar. See the relevant documentation on CVC5
+-- for details.
+--
+-- NB. Before you use this function, make sure to call
+--
+-- @
+--      setOption $ ProduceAbducts True
+-- @
+--
+-- to enable abduct generation.
+getAbduct :: Maybe String -> String -> SBool -> Query String
+getAbduct = Trans.getAbduct
+
+-- | Get the next abduct. Only call this after the first call to 'getAbduct' goes through. You can call
+-- it repeatedly to get a different abduct.
+getAbductNext :: Query String
+getAbductNext = Trans.getAbductNext
+
 -- | Retrieve assertions. Note you must have arranged for
 -- assertions to be available first via
 --
@@ -366,11 +389,9 @@ freshVar_ = Trans.freshVar_
 
 -- | Create a fresh variable in query mode. You should prefer
 -- creating input variables using 'Data.SBV.sBool', 'Data.SBV.sInt32', etc., which act
--- as primary inputs to the model and can be existential or universal.
--- Use 'freshVar' only in query mode for anonymous temporary variables.
--- Such variables are always existential. Note that 'freshVar' should hardly be
--- needed: Your input variables and symbolic expressions should suffice for
--- most major use cases.
+-- as primary inputs to the model. Use 'freshVar' only in query mode for anonymous temporary variables.
+-- Note that 'freshVar' should hardly be needed: Your input variables and symbolic expressions
+-- should suffice for -- most major use cases.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.Control.freshVar'
 freshVar :: SymVal a => String -> Query (SBV a)
@@ -391,6 +412,16 @@ freshArray_ = Trans.freshArray_
 freshArray :: (SymArray array, HasKind a, HasKind b) => String -> Maybe (SBV b) -> Query (array a b)
 freshArray = Trans.freshArray
 
+-- | Create a fresh lambda array
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.Control.freshLambdaArray'
+freshLambdaArray :: (HasKind a, HasKind b, Lambda (SymbolicT IO) (a -> b), SymArray array) => String -> (a -> b) -> Query (array a b)
+freshLambdaArray = Trans.freshLambdaArray
+
+-- | Create a fresh lambda array, unnamed
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.Control.freshLambdaArray_'
+freshLambdaArray_ :: (HasKind a, HasKind b, Lambda (SymbolicT IO) (a -> b), SymArray array) => (a -> b) -> Query (array a b)
+freshLambdaArray_ = Trans.freshLambdaArray_
+
 -- | If 'Data.SBV.verbose' is 'True', print the message, useful for debugging messages
 -- in custom queries. Note that 'Data.SBV.redirectVerbose' will be respected: If a
 -- file redirection is given, the output will go to the file.
@@ -438,7 +469,7 @@ getUninterpretedValue = Trans.getUninterpretedValue
 -- | Get the value of an uninterpreted function, as a list of domain, value pairs.
 -- The final value is the "else" clause, i.e., what the function maps values outside
 -- of the domain of the first list.
-getFunction :: (SymVal a, SymVal r, Trans.SMTFunction fun a r) => fun -> Query ([(a, r)], r)
+getFunction :: (SymVal a, SymVal r, Trans.SMTFunction fun a r) => fun -> Query (Either String ([(a, r)], r))
 getFunction = Trans.getFunction
 
 -- | Get the value of a term. If the kind is Real and solver supports decimal approximations,
@@ -449,13 +480,13 @@ getValueCV :: Maybe Int -> SV -> Query CV
 getValueCV = Trans.getValueCV
 
 -- | Get the value of an uninterpreted value
-getUICVal :: Maybe Int -> (String, SBVType) -> Query CV
+getUICVal :: Maybe Int -> (String, (Maybe [String], SBVType)) -> Query CV
 getUICVal = Trans.getUICVal
 
 -- | Get the value of an uninterpreted function as an association list
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.Control.getUIFunCVAssoc'
-getUIFunCVAssoc :: Maybe Int -> (String, SBVType) -> Query ([([CV], CV)], CV)
+getUIFunCVAssoc :: Maybe Int -> (String, (Maybe [String], SBVType)) -> Query (Either String ([([CV], CV)], CV))
 getUIFunCVAssoc = Trans.getUIFunCVAssoc
 
 -- | Check for satisfiability.
