@@ -21,7 +21,9 @@
 {-# LANGUAGE PatternGuards         #-}
 {-# LANGUAGE ScopedTypeVariables   #-}
 {-# LANGUAGE TypeApplications      #-}
+{-# LANGUAGE TypeOperators         #-}
 {-# LANGUAGE TypeFamilies          #-}
+{-# LANGUAGE UndecidableInstances  #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -48,7 +50,7 @@ module Data.SBV.Core.Data
  , sbvToSV, sbvToSymSV, forceSVArg
  , SBVExpr(..), newExpr
  , cache, Cached, uncache, uncacheAI, HasKind(..)
- , Op(..), PBOp(..), FPOp(..), StrOp(..), SeqOp(..), RegExp(..), NamedSymVar(..), getTableIndex
+ , Op(..), PBOp(..), FPOp(..), StrOp(..), RegExOp(..), SeqOp(..), RegExp(..), NamedSymVar(..), OvOp(..), getTableIndex
  , SBVPgm(..), Symbolic, runSymbolic, State, getPathCondition, extendPathCondition
  , inSMTMode, SBVRunMode(..), Kind(..), Outputtable(..), Result(..)
  , SolverContext(..), internalVariable, internalConstraint, isCodeGenMode
@@ -59,25 +61,29 @@ module Data.SBV.Core.Data
  , extractSymbolicSimulationState
  , SMTScript(..), Solver(..), SMTSolver(..), SMTResult(..), SMTModel(..), SMTConfig(..)
  , OptimizeStyle(..), Penalty(..), Objective(..)
- , QueryState(..), QueryT(..), SMTProblem(..)
+ , QueryState(..), QueryT(..), SMTProblem(..), Constraint(..), Lambda(..), Forall(..), Exists(..), ExistsUnique(..), ForallN(..), ExistsN(..)
+ , QuantifiedBool(..), EqSymbolic(..), QNot(..), Skolemize(skolemize, taggedSkolemize)
  ) where
 
-import GHC.TypeLits
+import GHC.TypeLits (KnownNat, Nat, Symbol, KnownSymbol, symbolVal, AppendSymbol)
 
-import GHC.Generics (Generic)
 import GHC.Exts     (IsList(..))
 
 import Control.DeepSeq        (NFData(..))
-import Control.Monad.Trans    (liftIO)
+import Control.Monad          (void, replicateM)
+import Control.Monad.Trans    (liftIO, MonadIO)
 import Data.Int               (Int8, Int16, Int32, Int64)
 import Data.Word              (Word8, Word16, Word32, Word64)
 import Data.List              (elemIndex)
 import Data.Maybe             (fromMaybe)
 
+import Data.Kind (Type)
 import Data.Proxy
 import Data.Typeable          (Typeable)
 
-import qualified Data.Generics as G    (Data(..))
+import GHC.Generics (Generic, U1(..), M1(..), (:*:)(..), K1(..), (:+:)(..))
+import qualified GHC.Generics  as G
+import qualified Data.Generics as G (Data(..))
 
 import qualified Data.IORef         as R    (readIORef)
 import qualified Data.IntMap.Strict as IMap (size, insert)
@@ -292,7 +298,7 @@ x .~| y = sNot (x .|| y)
 -- | Symbolic implication
 infixr 1 .=>
 (.=>) :: SBool -> SBool -> SBool
-x .=> y = sNot x .|| y
+SBV x .=> SBV y = SBV (x `svImplies` y)
 -- NB. Do *not* try to optimize @x .=> x = True@ here! If constants go through, it'll get simplified.
 -- The case "x .=> x" can hit is extremely rare, and the getAllSatResult function relies on this
 -- trick to generate constraints in the unlucky case of ui-function models.
@@ -300,7 +306,7 @@ x .=> y = sNot x .|| y
 -- | Symbolic boolean equivalence
 infixr 1 .<=>
 (.<=>) :: SBool -> SBool -> SBool
-x .<=> y = (x .&& y) .|| (sNot x .&& sNot y)
+SBV x .<=> SBV y = SBV (x `svEqual` y)
 
 -- | Conversion from 'Bool' to 'SBool'
 fromBool :: Bool -> SBool
@@ -403,41 +409,132 @@ sbvToSymSV sbv = do
         st <- symbolicEnv
         liftIO $ sbvToSV st sbv
 
+-- | Values that we can turn into a constraint
+class MonadSymbolic m => Constraint m a where
+  mkConstraint :: State -> a -> m ()
+
+-- | Base case: simple booleans
+instance MonadSymbolic m => Constraint m SBool where
+  mkConstraint _ out = void $ output out
+
+-- | An existential symbolic variable, used in building quantified constraints. The name
+-- attached via the symbol is used during skolemization to create a skolem-function name
+-- when this variable is eliminated.
+newtype Exists (nm :: Symbol) a = Exists (SBV a)
+
+-- | An existential unique symbolic variable, used in building quantified constraints. The name
+-- attached via the symbol is used during skolemization. It's split into two extra names, suffixed
+-- @_eu1@ and @_eu2@, to name the universals in the equivalent formula:
+-- \(\exists! x\,P(x)\Leftrightarrow \exists x\,P(x) \land \forall x_{eu1} \forall x_{eu2} (P(x_{eu1}) \land P(x_{eu2}) \Rightarrow x_{eu1} = x_{eu2}) \)
+newtype ExistsUnique (nm :: Symbol) a = ExistsUnique (SBV a)
+
+-- | A universal symbolic variable, used in building quantified constraints. The name attached via the symbol is used
+-- during skolemization. It names the corresponding argument to the skolem-functions within the scope of this quantifier.
+newtype Forall (nm :: Symbol) a = Forall (SBV a)
+
+-- | Exactly @n@ existential symbolic variables, used in building quantified constraints. The name attached
+-- will be prefixed in front of @_1@, @_2@, ..., @_n@ to form the names of the variables.
+newtype ExistsN (n :: Nat) (nm :: Symbol) a = ExistsN [SBV a]
+
+-- | Exactly @n@ universal symbolic variables, used in in building quantified constraints. The name attached
+-- will be prefixed in front of @_1@, @_2@, ..., @_n@ to form the names of the variables.
+newtype ForallN (n :: Nat) (nm :: Symbol) a = ForallN [SBV a]
+
+-- | make a quantifier argument in the given state
+mkQArg :: forall m a. (HasKind a, MonadIO m) => State -> Quantifier -> m (SBV a)
+mkQArg st q = do let k = kindOf (Proxy @a)
+                 sv <- liftIO $ quantVar q st k
+                 pure $ SBV $ SVal k (Right (cache (const (return sv))))
+
+-- | Functions of a single existential
+instance (SymVal a, Constraint m r) => Constraint m (Exists nm a -> r) where
+  mkConstraint st fn = mkQArg st EX >>= mkConstraint st . fn . Exists
+
+-- | Functions of a unique single existential
+instance (SymVal a, Constraint m r, EqSymbolic (SBV a), QuantifiedBool r) => Constraint m (ExistsUnique nm a -> r) where
+  mkConstraint st = mkConstraint st . rewriteExistsUnique
+
+-- | Functions of a number of existentials
+instance (KnownNat n, SymVal a, Constraint m r) => Constraint m (ExistsN n nm a -> r) where
+  mkConstraint st fn = replicateM (intOfProxy (Proxy @n)) (mkQArg st EX) >>= mkConstraint st . fn . ExistsN
+
+-- | Functions of a single universal
+instance (SymVal a, Constraint m r) => Constraint m (Forall nm a -> r) where
+  mkConstraint st fn = mkQArg st ALL >>= mkConstraint st . fn . Forall
+
+-- | Functions of a number of universals
+instance (KnownNat n, SymVal a, Constraint m r) => Constraint m (ForallN n nm a -> r) where
+  mkConstraint st fn = replicateM (intOfProxy (Proxy @n)) (mkQArg st ALL) >>= mkConstraint st . fn . ForallN
+
+-- | Values that we can turn into a lambda abstraction
+class MonadSymbolic m => Lambda m a where
+  mkLambda :: State -> a -> m ()
+
+-- | Base case, simple values
+instance MonadSymbolic m => Lambda m (SBV a) where
+  mkLambda _ out = void $ output out
+
+-- | Functions
+instance (SymVal a, Lambda m r) => Lambda m (SBV a -> r) where
+  mkLambda st fn = mkArg >>= mkLambda st . fn
+    where mkArg = do let k = kindOf (Proxy @a)
+                     sv <- liftIO $ lambdaVar st k
+                     pure $ SBV $ SVal k (Right (cache (const (return sv))))
+
+-- | A value that can be used as a quantified boolean
+class QuantifiedBool a where
+  -- | Turn a quantified boolean into a regular boolean. That is, this function turns an exists/forall quantified
+  -- formula to a simple boolean that can be used as a regular boolean value. An example is:
+  --
+  -- @
+  --   quantifiedBool $ \\(Forall x) (Exists y) -> y .> (x :: SInteger)
+  -- @
+  --
+  -- is equivalent to `sTrue`. You can think of this function as performing quantifier-elimination: It takes
+  -- a quantified formula, and reduces it to a simple boolean that is equivalent to it, but has no quantifiers.
+  quantifiedBool :: a -> SBool
+
+-- | Base case of quantification, simple booleans
+instance {-# OVERLAPPING #-} QuantifiedBool SBool where
+  quantifiedBool = id
+
 -- | Actions we can do in a context: Either at problem description
 -- time or while we are dynamically querying. 'Symbolic' and 'Query' are
 -- two instances of this class. Note that we use this mechanism
 -- internally and do not export it from SBV.
 class SolverContext m where
    -- | Add a constraint, any satisfying instance must satisfy this condition.
-   constrain :: SBool -> m ()
+   constrain :: QuantifiedBool a => a -> m ()
+
    -- | Add a soft constraint. The solver will try to satisfy this condition if possible, but won't if it cannot.
-   softConstrain :: SBool -> m ()
+   softConstrain :: QuantifiedBool a => a -> m ()
+
    -- | Add a named constraint. The name is used in unsat-core extraction.
-   namedConstraint :: String -> SBool -> m ()
+   namedConstraint :: QuantifiedBool a => String -> a -> m ()
+
    -- | Add a constraint, with arbitrary attributes.
-   constrainWithAttribute :: [(String, String)] -> SBool -> m ()
+   constrainWithAttribute :: QuantifiedBool a => [(String, String)] -> a -> m ()
+
    -- | Set info. Example: @setInfo ":status" ["unsat"]@.
    setInfo :: String -> [String] -> m ()
+
    -- | Set an option.
    setOption :: SMTOption -> m ()
+
    -- | Set the logic.
    setLogic :: Logic -> m ()
-   -- | Add a user specified axiom to the generated SMT-Lib file. The first argument is a mere
-   -- string, use for commenting purposes. The second argument is intended to hold the multiple-lines
-   -- of the axiom text as expressed in SMT-Lib notation. Note that we perform no checks on the axiom
-   -- itself, to see whether it's actually well-formed or is sensible by any means.
-   -- A separate formalization of SMT-Lib would be very useful here.
-   addAxiom :: String -> [String] -> m ()
+
    -- | Set a solver time-out value, in milli-seconds. This function
    -- essentially translates to the SMTLib call @(set-info :timeout val)@,
    -- and your backend solver may or may not support it! The amount given
    -- is in milliseconds. Also see the function 'Data.SBV.Control.timeOut' for finer level
    -- control of time-outs, directly from SBV.
    setTimeOut :: Integer -> m ()
+
    -- | Get the state associated with this context
    contextState :: m State
 
-   {-# MINIMAL constrain, softConstrain, namedConstraint, constrainWithAttribute, setOption, addAxiom, contextState #-}
+   {-# MINIMAL constrain, softConstrain, namedConstraint, constrainWithAttribute, setOption, contextState #-}
 
    -- time-out, logic, and info are  simply options in our implementation, so default implementation suffices
    setTimeOut t = setOption $ OptionKeyword ":timeout" [show t]
@@ -488,10 +585,13 @@ instance (Outputtable a, Outputtable b, Outputtable c, Outputtable d, Outputtabl
 class (HasKind a, Typeable a) => SymVal a where
   -- | Generalization of 'Data.SBV.mkSymVal'
   mkSymVal :: MonadSymbolic m => VarContext -> Maybe String -> m (SBV a)
+
   -- | Turn a literal constant to symbolic
   literal :: a -> SBV a
+
   -- | Extract a literal, from a CV representation
   fromCV :: CV -> a
+
   -- | Does it concretely satisfy the given predicate?
   isConcretely :: SBV a -> (a -> Bool) -> Bool
 
@@ -508,8 +608,11 @@ class (HasKind a, Typeable a) => SymVal a where
           k = constructUKind (undefined :: a)
 
   default literal :: Show a => a -> SBV a
-  literal x = let k@(KUserSort  _ conts) = kindOf x
-                  sx                     = show x
+  literal x = let k  = kindOf x
+                  sx = show x
+                  conts = case k of
+                           KUserSort _ cts -> cts
+                           _               -> Nothing
                   mbIdx = case conts of
                             Just xs -> sx `elemIndex` xs
                             Nothing -> Nothing
@@ -523,30 +626,6 @@ class (HasKind a, Typeable a) => SymVal a where
     | Just i <- unliteral s = p i
     | True                  = False
 
-  -- | Generalization of 'Data.SBV.forall'
-  forall :: MonadSymbolic m => String -> m (SBV a)
-  forall = mkSymVal (NonQueryVar (Just ALL)) . Just
-
-  -- | Generalization of 'Data.SBV.forall_'
-  forall_ :: MonadSymbolic m => m (SBV a)
-  forall_ = mkSymVal (NonQueryVar (Just ALL)) Nothing
-
-  -- | Generalization of 'Data.SBV.mkForallVars'
-  mkForallVars :: MonadSymbolic m => Int -> m [SBV a]
-  mkForallVars n = mapM (const forall_) [1 .. n]
-
-  -- | Generalization of 'Data.SBV.exists'
-  exists :: MonadSymbolic m => String -> m (SBV a)
-  exists = mkSymVal (NonQueryVar (Just EX)) . Just
-
-  -- | Generalization of 'Data.SBV.exists_'
-  exists_ :: MonadSymbolic m => m (SBV a)
-  exists_ = mkSymVal (NonQueryVar (Just EX)) Nothing
-
-  -- | Generalization of 'Data.SBV.mkExistVars'
-  mkExistVars :: MonadSymbolic m => Int -> m [SBV a]
-  mkExistVars n = mapM (const exists_) [1 .. n]
-
   -- | Generalization of 'Data.SBV.free'
   free :: MonadSymbolic m => String -> m (SBV a)
   free = mkSymVal (NonQueryVar Nothing) . Just
@@ -608,25 +687,31 @@ instance (Random a, SymVal a) => Random (SBV a) where
 -- referential transparency. See https://github.com/LeventErkok/sbv/issues/553 for details.
 class SymArray array where
   -- | Generalization of 'Data.SBV.newArray_'
-  newArray_      :: (MonadSymbolic m, HasKind a, HasKind b) => Maybe (SBV b) -> m (array a b)
+  newArray_ :: (MonadSymbolic m, HasKind a, HasKind b) => Maybe (SBV b) -> m (array a b)
+
   -- | Generalization of 'Data.SBV.newArray'
-  newArray       :: (MonadSymbolic m, HasKind a, HasKind b) => String -> Maybe (SBV b) -> m (array a b)
+  newArray  :: (MonadSymbolic m, HasKind a, HasKind b) => String -> Maybe (SBV b) -> m (array a b)
+
   -- | Create a literal array
-  sListArray     :: (HasKind a, SymVal b) => b -> [(SBV a, SBV b)] -> array a b
+  sListArray :: (HasKind a, SymVal b) => b -> [(SBV a, SBV b)] -> array a b
+
   -- | Read the array element at @a@
-  readArray      :: array a b -> SBV a -> SBV b
+  readArray :: array a b -> SBV a -> SBV b
+
   -- | Update the element at @a@ to be @b@
-  writeArray     :: SymVal b => array a b -> SBV a -> SBV b -> array a b
+  writeArray :: SymVal b => array a b -> SBV a -> SBV b -> array a b
+
   -- | Merge two given arrays on the symbolic condition
   -- Intuitively: @mergeArrays cond a b = if cond then a else b@.
   -- Merging pushes the if-then-else choice down on to elements
-  mergeArrays    :: SymVal b => SBV Bool -> array a b -> array a b -> array a b
+  mergeArrays :: SymVal b => SBV Bool -> array a b -> array a b -> array a b
+
   -- | Internal function, not exported to the user
-  newArrayInState :: (HasKind a, HasKind b) => Maybe String -> Maybe (SBV b) -> State -> IO (array a b)
+  newArrayInState :: (HasKind a, HasKind b) => Maybe String -> Either (Maybe (SBV b)) String -> State -> IO (array a b)
 
   {-# MINIMAL readArray, writeArray, mergeArrays, ((newArray_, newArray) | newArrayInState), sListArray #-}
-  newArray_   mbVal = symbolicEnv >>= liftIO . newArrayInState Nothing   mbVal
-  newArray nm mbVal = symbolicEnv >>= liftIO . newArrayInState (Just nm) mbVal
+  newArray_   mbVal = symbolicEnv >>= liftIO . newArrayInState Nothing   (Left mbVal)
+  newArray nm mbVal = symbolicEnv >>= liftIO . newArrayInState (Just nm) (Left mbVal)
 
   -- Despite our MINIMAL pragma and default implementations for newArray_ and
   -- newArray, we must provide a dummy implementation for newArrayInState:
@@ -665,15 +750,243 @@ instance SymArray SArray where
 
                            iSV <- sbvToSV st iVal
 
-                           let upd  = IMap.insert (unArrayIndex k) ("array_" ++ show k, ks, ArrayFree (Just iSV))
+                           let upd  = IMap.insert (unArrayIndex k) ("array_" ++ show k, ks, ArrayFree (Left (Just iSV)))
 
                            k `seq` modifyState st rArrayMap upd $ modifyIncState st rNewArrs upd
                            return k
 
-  newArrayInState :: forall a b. (HasKind a, HasKind b) => Maybe String -> Maybe (SBV b) -> State -> IO (SArray a b)
-  newArrayInState mbNm mbVal st = do mapM_ (registerKind st) [aknd, bknd]
-                                     SArray <$> newSArr st (aknd, bknd) (mkNm mbNm) (unSBV <$> mbVal)
+  newArrayInState :: forall a b. (HasKind a, HasKind b) => Maybe String -> Either (Maybe (SBV b)) String -> State -> IO (SArray a b)
+  newArrayInState mbNm eiVal st = do mapM_ (registerKind st) [aknd, bknd]
+                                     SArray <$> newSArr st (aknd, bknd) (mkNm mbNm) (either (Left . (unSBV <$>)) Right eiVal)
      where mkNm Nothing   t = "array_" ++ show t
            mkNm (Just nm) _ = nm
            aknd = kindOf (Proxy @a)
            bknd = kindOf (Proxy @b)
+
+-- | Symbolic Equality. Note that we can't use Haskell's 'Eq' class since Haskell insists on returning Bool
+-- Comparing symbolic values will necessarily return a symbolic value.
+--
+-- Minimal complete definition: None, if the type is instance of @Generic@. Otherwise '(.==)'.
+infix 4 .==, ./=, .===, ./==
+class EqSymbolic a where
+  -- | Symbolic equality.
+  (.==) :: a -> a -> SBool
+  -- | Symbolic inequality.
+  (./=) :: a -> a -> SBool
+  -- | Strong equality. On floats ('SFloat'/'SDouble'), strong equality is object equality; that
+  -- is @NaN == NaN@ holds, but @+0 == -0@ doesn't. On other types, (.===) is simply (.==).
+  -- Note that (.==) is the /right/ notion of equality for floats per IEEE754 specs, since by
+  -- definition @+0 == -0@ and @NaN@ equals no other value including itself. But occasionally
+  -- we want to be stronger and state @NaN@ equals @NaN@ and @+0@ and @-0@ are different from
+  -- each other. In a context where your type is concrete, simply use `Data.SBV.fpIsEqualObject`. But in
+  -- a polymorphic context, use the strong equality instead.
+  --
+  -- NB. If you do not care about or work with floats, simply use (.==) and (./=).
+  (.===) :: a -> a -> SBool
+  -- | Negation of strong equality. Equaivalent to negation of (.===) on all types.
+  (./==) :: a -> a -> SBool
+
+  -- | Returns (symbolic) 'sTrue' if all the elements of the given list are different.
+  distinct :: [a] -> SBool
+
+  -- | Returns (symbolic) `sTrue` if all the elements of the given list are different. The second
+  -- list contains exceptions, i.e., if an element belongs to that set, it will be considered
+  -- distinct regardless of repetition.
+  distinctExcept :: [a] -> [a] -> SBool
+
+  -- | Returns (symbolic) 'sTrue' if all the elements of the given list are the same.
+  allEqual :: [a] -> SBool
+
+  -- | Symbolic membership test.
+  sElem    :: a -> [a] -> SBool
+
+  -- | Symbolic negated membership test.
+  sNotElem :: a -> [a] -> SBool
+
+  x ./=  y = sNot (x .==  y)
+  x .=== y = x .== y
+  x ./== y = sNot (x .=== y)
+
+  allEqual []     = sTrue
+  allEqual (x:xs) = sAll (x .==) xs
+
+  -- Default implementation of 'distinct'. Note that we override
+  -- this method for the base types to generate better code.
+  distinct []     = sTrue
+  distinct (x:xs) = sAll (x ./=) xs .&& distinct xs
+
+  -- Default implementation of 'distinctExcept'. Note that we override
+  -- this method for the base types to generate better code.
+  distinctExcept es ignored = go es
+    where isIgnored = (`sElem` ignored)
+
+          go []     = sTrue
+          go (x:xs) = let xOK  = isIgnored x .|| sAll (\y -> isIgnored y .|| x ./= y) xs
+                      in xOK .&& go xs
+
+  x `sElem`    xs = sAny (.== x) xs
+  x `sNotElem` xs = sNot (x `sElem` xs)
+
+  -- Default implementation for '(.==)' if the type is 'Generic'
+  default (.==) :: (G.Generic a, GEqSymbolic (G.Rep a)) => a -> a -> SBool
+  (.==) = symbolicEqDefault
+
+-- | Default implementation of symbolic equality, when the underlying type is generic
+-- Not exported, used with automatic deriving.
+symbolicEqDefault :: (G.Generic a, GEqSymbolic (G.Rep a)) => a -> a -> SBool
+symbolicEqDefault x y = symbolicEq (G.from x) (G.from y)
+
+-- | Not exported, used for implementing generic equality.
+class GEqSymbolic f where
+  symbolicEq :: f a -> f a -> SBool
+
+{-
+ - N.B. A V1 instance like the below would be wrong!
+ - Why? Because in SBV, we use empty data to mean "uninterpreted" sort; not
+ - something that has no constructors. Perhaps that was a bad design
+ - decision. So, do not allow equality checking of such values.
+instance GEqSymbolic V1 where
+  symbolicEq _ _ = sTrue
+-}
+
+instance GEqSymbolic U1 where
+  symbolicEq _ _ = sTrue
+
+instance (EqSymbolic c) => GEqSymbolic (K1 i c) where
+  symbolicEq (K1 x) (K1 y) = x .== y
+
+instance (GEqSymbolic f) => GEqSymbolic (M1 i c f) where
+  symbolicEq (M1 x) (M1 y) = symbolicEq x y
+
+instance (GEqSymbolic f, GEqSymbolic g) => GEqSymbolic (f :*: g) where
+  symbolicEq (x1 :*: y1) (x2 :*: y2) = symbolicEq x1 x2 .&& symbolicEq y1 y2
+
+instance (GEqSymbolic f, GEqSymbolic g) => GEqSymbolic (f :+: g) where
+  symbolicEq (L1 l) (L1 r) = symbolicEq l r
+  symbolicEq (R1 l) (R1 r) = symbolicEq l r
+  symbolicEq (L1 _) (R1 _) = sFalse
+  symbolicEq (R1 _) (L1 _) = sFalse
+
+-- | A class of values that can be skolemized. Note that we don't export this class. Use
+-- the 'skolemize' function instead.
+class Skolemize a where
+  type SkolemsTo a :: Type
+  skolem :: String -> [(SVal, String)] -> a -> SkolemsTo a
+
+  -- | Skolemization. For any formula, skolemization gives back an equisatisfiable formula that
+  -- has no existential quantifiers in it. You have to provide enough names for all the
+  -- existentials in the argument. (Extras OK, so you can pass an infinite list if you like.)
+  -- The names should be distinct, and also different from any other uninterpreted name
+  -- you might have elsewhere.
+  skolemize :: (Constraint Symbolic (SkolemsTo a), Skolemize a) => a -> SkolemsTo a
+  skolemize = skolem "" []
+
+  -- | If you use the same names for skolemized arguments in different functions, they will
+  -- collide; which is undesirable. Unfortunately there's no easy way for SBV to detect this.
+  -- In such cases, use 'taggedSkolemize' to add a scope to the skolem-function names generated.
+  taggedSkolemize :: (Constraint Symbolic (SkolemsTo a), Skolemize a) => String -> a -> SkolemsTo a
+  taggedSkolemize scope = skolem (scope ++ "_") []
+
+-- | Base case; pure symbolic values
+instance Skolemize (SBV a) where
+  type SkolemsTo (SBV a) = SBV a
+  skolem _ _ = id
+
+-- | Skolemize over a universal quantifier
+instance (KnownSymbol nm, Skolemize r) => Skolemize (Forall nm a -> r) where
+  type SkolemsTo (Forall nm a -> r) = Forall nm a -> SkolemsTo r
+  skolem scope args f arg@(Forall a) = skolem scope (args ++ [(unSBV a, symbolVal (Proxy @nm))]) (f arg)
+
+-- | Skolemize over a number of universal quantifiers
+instance (KnownSymbol nm, Skolemize r) => Skolemize (ForallN n nm a -> r) where
+  type SkolemsTo (ForallN n nm a -> r) = ForallN n nm a -> SkolemsTo r
+  skolem scope args f arg@(ForallN xs) = skolem scope (args ++ zipWith grab xs [(1::Int)..]) (f arg)
+    where pre = symbolVal (Proxy @nm)
+          grab x i = (unSBV x, pre ++ "_" ++ show i)
+
+-- | Skolemize over an existential quantifier
+instance (HasKind a, KnownSymbol nm, Skolemize r) => Skolemize (Exists nm a -> r) where
+  type SkolemsTo (Exists nm a -> r) = SkolemsTo r
+  skolem scope args f = skolem scope args (f (Exists skolemized))
+    where skolemized = SBV $ svUninterpretedNamedArgs (kindOf (Proxy @a)) (scope ++ symbolVal (Proxy @nm)) UINone args
+
+-- | Skolemize over a number of existential quantifiers
+instance (HasKind a, KnownNat n, KnownSymbol nm, Skolemize r) => Skolemize (ExistsN n nm a -> r) where
+  type SkolemsTo (ExistsN n nm a -> r) = SkolemsTo r
+  skolem scope args f = skolem scope args (f (ExistsN skolemized))
+    where need   = intOfProxy (Proxy @n)
+          prefix = symbolVal (Proxy @nm)
+          fs     = [prefix ++ "_" ++ show i | i <- [1 .. need]]
+          skolemized = [SBV $ svUninterpretedNamedArgs (kindOf (Proxy @a)) (scope ++ n) UINone args | n <- fs]
+
+-- | Skolemize over a unique existential quantifier
+instance (  HasKind a
+          , EqSymbolic (SBV a)
+          , KnownSymbol nm
+          , QuantifiedBool r
+          , Skolemize (Forall (AppendSymbol nm "_eu1") a -> Forall (AppendSymbol nm "_eu2") a -> SBool)
+         ) => Skolemize (ExistsUnique nm a -> r) where
+  type SkolemsTo (ExistsUnique nm a -> r) =  Forall (AppendSymbol nm "_eu1") a
+                                          -> Forall (AppendSymbol nm "_eu2") a
+                                          -> SBool
+  skolem scope args f = skolem scope args (rewriteExistsUnique f (Exists skolemized))
+    where skolemized = SBV $ svUninterpretedNamedArgs (kindOf (Proxy @a)) (scope ++ symbolVal (Proxy @nm)) UINone args
+
+-- | Class of things that we can logically negate
+class QNot a where
+  type NegatesTo a :: Type
+  -- | Negation of a quantified formula. This operation essentially lifts 'sNot' to quantified formulae.
+  -- Note that you can achieve the same using @'sNot' . 'quantifiedBool'@, but that will hide the
+  -- quantifiers, so prefer this version if you want to keep them around.
+  qNot :: a -> NegatesTo a
+
+-- | Base case; pure symbolic boolean
+instance QNot SBool where
+  type NegatesTo SBool = SBool
+  qNot = sNot
+
+-- | Negate over a universal quantifier. Switches to existential.
+instance QNot r => QNot (Forall nm a -> r) where
+  type NegatesTo (Forall nm a -> r) = Exists nm a -> NegatesTo r
+  qNot f (Exists a) = qNot (f (Forall a))
+
+-- | Negate over a number of universal quantifiers
+instance QNot r => QNot (ForallN nm n a -> r) where
+  type NegatesTo (ForallN nm n a -> r) = ExistsN nm n a -> NegatesTo r
+  qNot f (ExistsN xs) = qNot (f (ForallN xs))
+
+-- | Negate over an existential quantifier. Switches to universal.
+instance QNot r => QNot (Exists nm a -> r) where
+  type NegatesTo (Exists nm a -> r) = Forall nm a -> NegatesTo r
+  qNot f (Forall a) = qNot (f (Exists a))
+
+-- | Negate over a number of existential quantifiers
+instance QNot r => QNot (ExistsN nm n a -> r) where
+  type NegatesTo (ExistsN nm n a -> r) = ForallN nm n a -> NegatesTo r
+  qNot f (ForallN xs) = qNot (f (ExistsN xs))
+
+-- | Negate over an unique existential quantifier
+instance (QNot r, QuantifiedBool r, EqSymbolic (SBV a)) => QNot (ExistsUnique nm a -> r) where
+  type NegatesTo (ExistsUnique nm a -> r) =  Forall nm a
+                                          -> Exists (AppendSymbol nm "_eu1") a
+                                          -> Exists (AppendSymbol nm "_eu2") a
+                                          -> SBool
+  qNot = qNot . rewriteExistsUnique
+
+-- | Get rid of exists unique.
+rewriteExistsUnique :: ( QuantifiedBool b                 -- If b can be turned into a boolean
+                       , EqSymbolic (SBV a)               -- If we can do equality on symbolic a's
+                       )                                  -- THEN
+                    => (ExistsUnique nm a -> b)           -- Given an unique-existential, we can
+                    -> Exists nm a                        -- Turn it into an existential
+                    -> Forall (AppendSymbol nm "_eu1") a  -- A universal
+                    -> Forall (AppendSymbol nm "_eu2") a  -- Another universal
+                    -> SBool                                  -- Making sure given holds, and if both univers hold, they're the same
+rewriteExistsUnique f (Exists x) (Forall x1) (Forall x2) = fx .&& unique
+  where fx    = quantifiedBool $ f (ExistsUnique x)
+        fx1   = f (ExistsUnique x1)
+        fx2   = f (ExistsUnique x2)
+
+        bothHolds  = quantifiedBool fx1 .&& quantifiedBool fx2
+        mustEqual  = x1 .== x2
+        unique     = bothHolds .=> mustEqual
