@@ -23,11 +23,11 @@ module Data.SBV.Core.Operations
   -- ** Basic operations
   , svPlus, svTimes, svMinus, svUNeg, svAbs
   , svDivide, svQuot, svRem, svQuotRem
-  , svEqual, svNotEqual, svStrongEqual, svSetEqual
+  , svEqual, svNotEqual, svStrongEqual, svImplies, svSetEqual
   , svLessThan, svGreaterThan, svLessEq, svGreaterEq, svStructuralLessThan
   , svAnd, svOr, svXOr, svNot
   , svShl, svShr, svRol, svRor
-  , svExtract, svJoin
+  , svExtract, svJoin, svZeroExtend, svSignExtend
   , svIte, svLazyIte, svSymbolicMerge
   , svSelect
   , svSign, svUnsign, svSetBit, svWordFromBE, svWordFromLE
@@ -41,7 +41,7 @@ module Data.SBV.Core.Operations
   , svBarrelRotateLeft, svBarrelRotateRight
   , svBlastLE, svBlastBE
   , svAddConstant, svIncrement, svDecrement
-  , svFloatAsSWord32, svDoubleAsSWord64, svFloatingPointAsSWord 
+  , svFloatAsSWord32, svDoubleAsSWord64, svFloatingPointAsSWord
   -- ** Basic array operations
   , SArr(..), readSArr, writeSArr, mergeSArr, newSArr, eqSArr
   -- Utils
@@ -49,8 +49,9 @@ module Data.SBV.Core.Operations
   )
   where
 
+import Prelude hiding (Foldable(..))
 import Data.Bits (Bits(..))
-import Data.List (genericIndex, genericLength, genericTake)
+import Data.List (genericIndex, genericLength, genericTake, foldr, length, foldl')
 
 import qualified Data.IORef         as R    (readIORef)
 import qualified Data.IntMap.Strict as IMap (size, insert)
@@ -298,6 +299,24 @@ svEqual a b
   | True
   = liftSym2B (mkSymOpSC (eqOptBool Equal trueSV) Equal) rationalCheck (==) (==) (==) (==) (==) (==) (==) (==) (==) (==) (==) (==) (==) a b
 
+-- | Implication. Only for booleans.
+svImplies :: SVal -> SVal -> SVal
+svImplies a b
+  |    isConcreteZero a -- false implies everything
+    || isConcreteOne  b -- true is implied by everything
+  = svTrue
+  | any (\x -> kindOf x /= KBool) [a, b]
+  = bad
+  | True
+  = liftSym2B (mkSymOpSC (eqOpt trueSV) Implies) rationalCheck bad imp bad bad bad bad bad bad bad bad bad bad bad a b
+  where bad = error $ "Data.SBV.svImplies: Unexpected arguments: " ++ show (a, kindOf a, b, kindOf b)
+        imp :: Integer -> Integer -> Bool
+        imp 0 0 = True
+        imp 0 1 = True
+        imp 1 0 = False
+        imp 1 1 = True
+        imp x y = error $ "Data.SBV.svImplies: Called on unreduced arguments: " ++ show (x, y, a, kindOf a, b, kindOf b)
+
 -- | Inequality.
 svNotEqual :: SVal -> SVal -> SVal
 svNotEqual a b
@@ -522,11 +541,21 @@ svExtract _ _ _ = error "extract: non-bitvector/float type"
 
 -- | Join two words, by concatenating
 svJoin :: SVal -> SVal -> SVal
-svJoin x@(SVal (KBounded s i) a) y@(SVal (KBounded _ j) b)
+svJoin x@(SVal (KBounded s i) a) y@(SVal (KBounded s' j) b)
+  | s /= s'
+  = error $ "svJoin: received differently signed values: " ++ show (x, y)
   | i == 0 = y
   | j == 0 = x
   | Left (CV _ (CInteger m)) <- a, Left (CV _ (CInteger n)) <- b
-  = SVal k (Left $! CV k (CInteger (m `shiftL` j .|. n)))
+  = let val
+         | s -- signed, arithmetic doesn't work; blast and come back
+         = let xbits = [m `testBit` xi | xi <- [0 .. i-1]]
+               ybits = [n `testBit` yi | yi <- [0 .. j-1]]
+               rbits = zip [0..] (ybits ++ xbits)
+           in foldl' (\acc (idx, set) -> if set then setBit acc idx else acc) 0 rbits
+         | True -- unsigned, go fast
+         = m `shiftL` j .|. n
+    in SVal k (Left $! normCV (CV k (CInteger val)))
   | True
   = SVal k (Right (cache z))
   where
@@ -536,6 +565,37 @@ svJoin x@(SVal (KBounded s i) a) y@(SVal (KBounded _ j) b)
               newExpr st k (SBVApp Join [xsw, ysw])
 svJoin _ _ = error "svJoin: non-bitvector type"
 
+-- | Zero-extend by given number of bits.
+svZeroExtend :: Int -> SVal -> SVal
+svZeroExtend = svExtend True ZeroExtend
+
+-- | Sign-extend by given number of bits.
+svSignExtend :: Int -> SVal -> SVal
+svSignExtend = svExtend False SignExtend
+
+svExtend :: Bool -> (Int -> Op) -> Int -> SVal -> SVal
+svExtend isZeroExtend extender i x@(SVal (KBounded s sz) a)
+  | i < 0
+  = error $ "svExtend: Received negative extension amount: " ++ show i
+  | i == 0
+  = x
+  | Left (CV _ (CInteger cv)) <- a
+  = SVal k' (Left (normCV (CV k' (CInteger (replBit (not isZeroExtend && (cv `testBit` (sz-1))) cv)))))
+  | True
+  = SVal k' (Right (cache z))
+  where k' = KBounded s (sz+i)
+        z st = do xsw <- svToSV st x
+                  newExpr st k' (SBVApp (extender i) [xsw])
+
+        replBit :: Bool -> Integer -> Integer
+        replBit b = go sz
+          where stop = sz + i
+                go k v | k == stop = v
+                       | b         = go (k+1) (v `setBit`   k)
+                       | True      = go (k+1) (v `clearBit` k)
+
+svExtend _ _ _ _ = error "svExtend: non-bitvector type"
+
 -- | If-then-else. This one will force branches.
 svIte :: SVal -> SVal -> SVal -> SVal
 svIte t a b = svSymbolicMerge (kindOf a) True t a b
@@ -978,13 +1038,15 @@ mergeSArr t (SArr ainfo a) (SArr _ b) = SArr ainfo $ cache h
                   return k
 
 -- | Create a named new array
-newSArr :: State -> (Kind, Kind) -> (Int -> String) -> Maybe SVal -> IO SArr
-newSArr st ainfo mkNm mbDef = do
+newSArr :: State -> (Kind, Kind) -> (Int -> String) -> Either (Maybe SVal) String -> IO SArr
+newSArr st ainfo mkNm mbVal = do
     amap <- R.readIORef $ rArrayMap st
 
-    mbSWDef <- case mbDef of
-                 Nothing -> return Nothing
-                 Just sv -> Just <$> svToSV st sv
+    mbSWDef <- case mbVal of
+                 Left mbSV -> case mbSV of
+                                Nothing -> pure (Left Nothing)
+                                Just sv -> Left . Just <$> svToSV st sv
+                 Right lam -> pure (Right lam)
 
     let i   = ArrayIndex $ IMap.size amap
         nm  = mkNm (unArrayIndex i)
@@ -1391,6 +1453,6 @@ svFloatingPointAsSWord fVal@(SVal kFrom@(KFP eb sb) _)
                              return n
 svFloatingPointAsSWord (SVal k _) = error $ "svFloatingPointAsSWord: non-float type: " ++ show k
 
-{-# ANN svIte     ("HLint: ignore Eta reduce" :: String)         #-}
-{-# ANN svLazyIte ("HLint: ignore Eta reduce" :: String)         #-}
-{-# ANN module    ("HLint: ignore Reduce duplication" :: String) #-}
+{- HLint ignore svIte     "Eta reduce"         -}
+{- HLint ignore svLazyIte "Eta reduce"         -}
+{- HLint ignore module    "Reduce duplication" -}
