@@ -46,6 +46,8 @@ abc = SMTSolver {
                               , supportsCustomQueries      = False
                               , supportsGlobalDecls        = False
                               , supportsDataTypes          = False
+                              , supportsFoldAndMap         = False
+                              , supportsSpecialRels        = False
                               , supportsDirectAccessors    = False
                               , supportsFlattenedModels    = Nothing
                               }
