@@ -48,6 +48,8 @@ cvc4 = SMTSolver {
                               , supportsCustomQueries      = True
                               , supportsGlobalDecls        = True
                               , supportsDataTypes          = True
+                              , supportsFoldAndMap         = False
+                              , supportsSpecialRels        = False
                               , supportsDirectAccessors    = True
                               , supportsFlattenedModels    = Nothing
                               }
