@@ -45,7 +45,7 @@ import qualified Control.Exception as C
 
 import Control.Concurrent (newEmptyMVar, takeMVar, putMVar, forkIO)
 import Control.DeepSeq    (NFData(..))
-import Control.Monad      (zipWithM)
+import Control.Monad      (zipWithM, mplus)
 import Data.Char          (isSpace)
 import Data.Maybe         (fromMaybe, isJust)
 import Data.Int           (Int8, Int16, Int32, Int64)
@@ -59,10 +59,12 @@ import Data.IORef (readIORef, writeIORef)
 
 import Data.Time          (getZonedTime, defaultTimeLocale, formatTime, diffUTCTime, getCurrentTime)
 
+import Data.Either(rights)
+
 import System.Directory   (findExecutable)
 import System.Environment (getEnv)
 import System.Exit        (ExitCode(..))
-import System.IO          (hClose, hFlush, hPutStrLn, hGetContents, hGetLine)
+import System.IO          (hClose, hFlush, hPutStrLn, hGetContents, hGetLine, hReady, hGetChar)
 import System.Process     (runInteractiveProcess, waitForProcess, terminateProcess)
 
 import qualified Data.Map.Strict as M
@@ -70,7 +72,7 @@ import qualified Data.Text       as T
 
 import Data.SBV.Core.AlgReals
 import Data.SBV.Core.Data
-import Data.SBV.Core.Symbolic (SMTEngine, State(..))
+import Data.SBV.Core.Symbolic (SMTEngine, State(..), mustIgnoreVar)
 import Data.SBV.Core.Concrete (showCV)
 import Data.SBV.Core.Kind     (showBaseKind, intOfProxy)
 
@@ -109,7 +111,6 @@ newtype SatResult = SatResult SMTResult
 
 -- | An 'Data.SBV.allSat' call results in a 'AllSatResult'
 data AllSatResult = AllSatResult { allSatMaxModelCountReached  :: Bool          -- ^ Did we reach the user given model count limit?
-                                 , allSatHasPrefixExistentials :: Bool          -- ^ Were there quantifiers in the problem (unique upto prefix existentials)
                                  , allSatSolverReturnedUnknown :: Bool          -- ^ Did the solver report unknown at the end?
                                  , allSatSolverReturnedDSat    :: Bool          -- ^ Did the solver report delta-satisfiable at the end?
                                  , allSatResults               :: [SMTResult]   -- ^ All satisfying models
@@ -167,16 +168,12 @@ instance Show SafeResult where
 -- The Show instance of AllSatResults.
 instance Show AllSatResult where
   show AllSatResult { allSatMaxModelCountReached  = l
-                    , allSatHasPrefixExistentials = e
                     , allSatSolverReturnedUnknown = u
                     , allSatSolverReturnedDSat    = d
                     , allSatResults               = xs
                     } = go (0::Int) xs
-    where warnings = case (e, u) of
-                       (False, False) -> ""
-                       (False, True)  -> " (Search stopped since solver has returned unknown.)"
-                       (True,  False) -> " (Unique up to prefix existentials.)"
-                       (True,  True)  -> " (Search stopped because solver has returned unknown, only prefix existentials were considered.)"
+    where warnings | u    = " (Search stopped since solver has returned unknown.)"
+                   | True = ""
 
           go c (s:ss) = let c'      = c+1
                             (ok, o) = sh c' s
@@ -420,10 +417,10 @@ class Modelable a where
   getModelObjectiveValue v r = v `M.lookup` getModelObjectives r
 
   -- | Extract model uninterpreted-functions
-  getModelUIFuns :: a -> M.Map String (SBVType, ([([CV], CV)], CV))
+  getModelUIFuns :: a -> M.Map String (SBVType, Either String ([([CV], CV)], CV))
 
   -- | Extract the value of an uninterpreted-function as an association list
-  getModelUIFunValue :: String -> a -> Maybe (SBVType, ([([CV], CV)], CV))
+  getModelUIFunValue :: String -> a -> Maybe (SBVType, Either String ([([CV], CV)], CV))
   getModelUIFunValue v r = v `M.lookup` getModelUIFuns r
 
 -- | Return all the models from an 'Data.SBV.allSat' call, similar to 'extractModel' but
@@ -507,7 +504,7 @@ parseModelOut m = case parseCVs [c | (_, c) <- modelAssocs m] of
 -- The arrange argument can sort the results in any way you like, if necessary.
 displayModels :: SatModel a => ([(Bool, a)] -> [(Bool, a)]) -> (Int -> (Bool, a) -> IO ()) -> AllSatResult -> IO Int
 displayModels arrange disp AllSatResult{allSatResults = ms} = do
-    let models = [a | Right a <- map (getModelAssignment . SatResult) ms]
+    let models = rights (map (getModelAssignment . SatResult) ms)
     inds <- zipWithM display (arrange models) [(1::Int)..]
     return $ last (0:inds)
   where display r i = disp i r >> return i
@@ -561,7 +558,7 @@ showModelDictionary warnEmpty includeEverything cfg allVars
         relevantVars  = filter (not . ignore) allVars
         ignore (T.pack -> s, _)
           | includeEverything = False
-          | True              = "__internal_sbv_" `T.isPrefixOf` s || isNonModelVar cfg (T.unpack s)
+          | True              = mustIgnoreVar cfg (T.unpack s)
 
         shM (s, RegularCV v) = let vs = shCV cfg v in ((length s, s), (vlength vs, vs))
         shM (s, other)       = let vs = show other in ((length s, s), (vlength vs, vs))
@@ -584,23 +581,26 @@ showModelDictionary warnEmpty includeEverything cfg allVars
         lTrimRight = length . dropWhile isSpace . reverse
 
 -- | Show an uninterpreted function
-showModelUI :: SMTConfig -> (String, (SBVType, ([([CV], CV)], CV))) -> String
-showModelUI cfg (nm, (SBVType ts, (defs, dflt))) = intercalate "\n" ["  " ++ l | l <- sig : map align body]
+showModelUI :: SMTConfig -> (String, (SBVType, Either String ([([CV], CV)], CV))) -> String
+showModelUI cfg (nm, (SBVType ts, interp))
+  = intercalate "\n" $ case interp of
+                         Left  e  -> ["  " ++ l | l <- [sig,  e]]
+                         Right ds -> ["  " ++ l | l <- sig : mkBody ds]
   where noOfArgs = length ts - 1
 
         sig      = nm ++ " :: " ++ intercalate " -> " (map showBaseKind ts)
 
-        ls       = map line defs
-        defLine  = (nm : replicate noOfArgs "_", scv dflt)
+        mkBody (defs, dflt) = map align body
+          where ls       = map line defs
+                defLine  = (nm : replicate noOfArgs "_", scv dflt)
+                body     = ls ++ [defLine]
 
-        body     = ls ++ [defLine]
+                colWidths = [maximum (0 : map length col) | col <- transpose (map fst body)]
 
-        colWidths = [maximum (0 : map length col) | col <- transpose (map fst body)]
+                resWidth  = maximum  (0 : map (length . snd) body)
 
-        resWidth  = maximum  (0 : map (length . snd) body)
-
-        align (xs, r) = unwords $ zipWith left colWidths xs ++ ["=", left resWidth r]
-           where left i x = take i (x ++ repeat ' ')
+                align (xs, r) = unwords $ zipWith left colWidths xs ++ ["=", left resWidth r]
+                   where left i x = take i (x ++ repeat ' ')
 
         -- NB. We'll ignore crackNum here. Seems to be overkill while displaying an
         -- uninterpreted function.
@@ -686,9 +686,9 @@ standardSolver config ctx pgm continuation = do
     rnf pgm `seq` pipeProcess config ctx exec opts pgm continuation
 
 -- | An internal type to track of solver interactions
-data SolverLine = SolverRegular   String  -- ^ All is well
-                | SolverTimeout   String  -- ^ Timeout expired
-                | SolverException String  -- ^ Something else went wrong
+data SolverLine = SolverRegular   String -- ^ All is well
+                | SolverTimeout   String -- ^ Timeout expired
+                | SolverException String -- ^ Something else went wrong
 
 -- | A variant of @readProcessWithExitCode@; except it deals with SBV continuations
 runSolver :: SMTConfig -> State -> FilePath -> [String] -> String -> (State -> IO a) -> IO a
@@ -698,6 +698,9 @@ runSolver cfg ctx execPath opts pgm continuation
 
           clean = preprocess (solver cfg)
 
+          -- the very first command we send
+          heartbeat = "(set-option :print-success true)"
+
       (send, ask, getResponseFromSolver, terminateSolver, cleanUp, pid) <- do
                 (inh, outh, errh, pid) <- runInteractiveProcess execPath opts Nothing Nothing
 
@@ -706,12 +709,23 @@ runSolver cfg ctx execPath opts pgm continuation
                                                 hFlush inh
                                                 recordTranscript (transcript cfg) $ Left (command, mbTimeOut)
 
+                    -- is this a set-command? Then we expect faster response; except for the heartbeat
+                    isSetCommand = maybe False chk
+                      where chk cmd = cmd /= heartbeat && "(set-option :" `isPrefixOf` cmd
+
                     -- Send a line, get a whole s-expr. We ignore the pathetic case that there might be a string with an unbalanced parentheses in it in a response.
                     ask :: Maybe Int -> String -> IO String
-                    ask mbTimeOut command =
-                                  -- solvers don't respond to empty lines or comments; we just pass back
-                                  -- success in these cases to keep the illusion of everything has a response
-                                  let cmd = dropWhile isSpace command
+                    ask mbTimeOutGiven command =
+                                  let -- If the command is a set-option call, make sure there's a timeout on it
+                                      -- This ensures that if we try to set an option before diagnostic-output
+                                      -- is redirected to stdout and the solver chokes, then we can catch it
+                                      mbTimeOut | isSetCommand (Just command) = Just 1000000
+                                                | True                        = mbTimeOutGiven
+
+                                      -- solvers don't respond to empty lines or comments; we just pass back
+                                      -- success in these cases to keep the illusion of everything has a response
+                                      cmd = dropWhile isSpace command
+
                                   in if null cmd || ";" `isPrefixOf` cmd
                                      then return "success"
                                      else do send mbTimeOut command
@@ -729,8 +743,9 @@ runSolver cfg ctx execPath opts pgm continuation
                                 return collated
 
                       where safeGetLine isFirst h =
-                                         let timeOutToUse | isFirst = mbTimeOut
-                                                          | True    = Just 5000000
+                                         let timeOutToUse | isSetCommand mbCommand = Just 1000000
+                                                          | isFirst                = mbTimeOut
+                                                          | True                   = Just 5000000
                                              timeOutMsg t | isFirst = "User specified timeout of " ++ showTimeoutValue t ++ " exceeded"
                                                           | True    = "A multiline complete response wasn't received before " ++ showTimeoutValue t ++ " exceeded"
 
@@ -751,13 +766,28 @@ runSolver cfg ctx execPath opts pgm continuation
                                                                                      then collect True sofar'
                                                                                      else return sofar'
 
+                                             -- Carefully grab things as they are ready. But don't block!
+                                             collectH handle = reverse <$> coll ""
+                                               where coll sofar = do b <- hReady handle
+                                                                     if b
+                                                                        then hGetChar handle >>= \c -> coll (c:sofar)
+                                                                        else pure sofar
+
+                                             -- grab the contents of a handle, and return it trimmed if any
+                                             grab handle = do mbCts <- (Just <$> collectH handle) `C.catch` (\(_ :: C.SomeException) -> pure Nothing)
+                                                              pure $ dropWhile isSpace <$> mbCts
+
                                          in case timeOutToUse of
                                               Nothing -> SolverRegular <$> getFullLine
                                               Just t  -> do r <- Timeout.timeout t getFullLine
                                                             case r of
                                                               Just l  -> return $ SolverRegular l
-                                                              Nothing -> return $ SolverTimeout $ timeOutMsg t
-
+                                                              Nothing -> do out <- grab outh
+                                                                            err <- grab errh
+                                                                            -- in this case, if we have something on out/err pass that back as regular
+                                                                            case err `mplus` out of
+                                                                              Just x | not (null x) -> pure $ SolverRegular x
+                                                                              _                     -> pure $ SolverTimeout (timeOutMsg t)
 
                             go isFirst i sofar = do
                                             errln <- safeGetLine isFirst outh `C.catch` (\(e :: C.SomeException) -> handleAsync e (return (SolverException (show e))))
@@ -796,6 +826,7 @@ runSolver cfg ctx execPath opts pgm continuation
                                                                                              }
 
                                               SolverTimeout e -> do terminateProcess pid -- NB. Do not *wait* for the process, just quit.
+
                                                                     C.throwIO SBVException { sbvExceptionDescription = "Timeout! " ++ e
                                                                                            , sbvExceptionSent        = mbCommand
                                                                                            , sbvExceptionExpected    = Nothing
@@ -912,8 +943,7 @@ runSolver cfg ctx execPath opts pgm continuation
                              let backend = name $ solver cfg
                              if not (supportsCustomQueries (capabilities (solver cfg)))
                                 then debug cfg ["** Skipping heart-beat for the solver " ++ show backend]
-                                else do let heartbeat = "(set-option :print-success true)"
-                                        r <- ask (Just 5000000) heartbeat  -- Give the solver 5s to respond, this should be plenty enough!
+                                else do r <- ask (Just 5000000) heartbeat  -- Give the solver 5s to respond, this should be plenty enough!
                                         case words r of
                                           ["success"]     -> debug cfg ["[GOOD] " ++ heartbeat]
                                           ["unsupported"] -> error $ unlines [ ""
