@@ -55,16 +55,15 @@ import Data.Proxy (Proxy(Proxy))
 import qualified Data.Set as Set
 
 import Data.SBV.Core.Data
-import Data.SBV.Core.Model    ((.==), (./=))
+import Data.SBV.Core.Model () -- instances only
 import Data.SBV.Core.Symbolic (SetOp(..))
 
 import qualified Data.Generics.Uniplate.Data as G
 
--- For doctest use only
---
 -- $setup
--- >>> import Data.SBV.Core.Model
--- >>> import Data.SBV.Provers.Prover
+-- >>> -- For doctest purposes only:
+-- >>> import Prelude hiding(null)
+-- >>> import Data.SBV hiding(complement)
 -- >>> :set -XScopedTypeVariables
 
 -- | Empty set.
@@ -150,8 +149,8 @@ complement ss
 --
 -- >>> prove $ \x (s :: SSet Integer) -> x `delete` (x `insert` s) .== s
 -- Falsifiable. Counter-example:
---   s0 =   2 :: Integer
---   s1 = {2} :: {Integer}
+--   s0 = 2 :: Integer
+--   s1 = U :: {Integer}
 --
 -- But the above is true if the element isn't in the set to start with:
 --
@@ -193,8 +192,8 @@ insert se ss
 --
 -- >>> prove $ \x (s :: SSet Integer) -> x `insert` (x `delete` s) .== s
 -- Falsifiable. Counter-example:
---   s0 =  2 :: Integer
---   s1 = {} :: {Integer}
+--   s0 =       2 :: Integer
+--   s1 = U - {2} :: {Integer}
 --
 -- But the above is true if the element is in the set to start with:
 --
@@ -312,7 +311,10 @@ isUniversal :: HasKind a => SSet a -> SBool
 isUniversal = isFull
 
 -- | Does the set have the given size? It implicitly asserts that the set
--- it is operating on is finite. Also see 'Data.SBV.Set.card'.
+-- it is operating on is finite. NB. Only z3 supported this call, and as
+-- discussed in http://github.com/Z3Prover/z3/issues/3854, recent versions
+-- of z3 doesn't support size calls either. So, you can only use this if you have
+-- a sufficiently old version of z3.
 --
 -- >>> prove $ \i -> hasSize (empty :: SSet Integer) i .== (i .== 0)
 -- Q.E.D.
@@ -320,14 +322,16 @@ isUniversal = isFull
 -- >>> sat $ \i -> hasSize (full :: SSet Integer) i
 -- Unsatisfiable
 --
--- ->>> prove $ \a b i j k -> hasSize (a :: SSet Integer) i .&& hasSize (b :: SSet Integer) j .&& hasSize (a `union` b) k .=> k .>= i `smax` j
--- Q.E.D.
+-- The following tests are commented out since z3 no longer supports size:
 --
--- ->>> prove $ \a b i j k -> hasSize (a :: SSet Integer) i .&& hasSize (b :: SSet Integer) j .&& hasSize (a `intersection` b) k .=> k .<= i `smin` j
--- Q.E.D.
+-- > >>> prove $ \a b i j k -> hasSize (a :: SSet Integer) i .&& hasSize (b :: SSet Integer) j .&& hasSize (a `union` b) k .=> k .>= i `smax` j
+-- > Q.E.D.
 --
--- ->>> prove $ \a k -> hasSize (a :: SSet Integer) k .=> k .>= 0
--- Q.E.D.
+-- > >>> prove $ \a b i j k -> hasSize (a :: SSet Integer) i .&& hasSize (b :: SSet Integer) j .&& hasSize (a `intersection` b) k .=> k .<= i `smin` j
+-- > Q.E.D.
+--
+-- > >>> prove $ \a k -> hasSize (a :: SSet Integer) k .=> k .>= 0
+-- > Q.E.D.
 hasSize :: (Ord a, SymVal a) => SSet a -> SInteger -> SBool
 hasSize sa si
   -- Case 1: Constant regular set, see if the size matches
@@ -546,9 +550,9 @@ True
 False
 >>> sat $ \(x::SSet (Maybe Integer)) y z -> distinct [x, y, z]
 Satisfiable. Model:
-  s0 = U - {Just 2} :: {Maybe Integer}
-  s1 =           {} :: {Maybe Integer}
-  s2 =            U :: {Maybe Integer}
+  s0 = {Just 3} :: {Maybe Integer}
+  s1 =       {} :: {Maybe Integer}
+  s2 =        U :: {Maybe Integer}
 
 However, if we compare two sets that are constructed as regular or in the complement
 form, we have to use a proof to establish equality:
