@@ -20,6 +20,8 @@
 -----------------------------------------------------------------------------
 
 {-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE TypeFamilies     #-}
+{-# LANGUAGE TypeOperators    #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -80,7 +82,7 @@ instance Show a => Show (InductionResult a) where
 
 -- | Induction engine, using the default solver. See "Documentation.SBV.Examples.ProofTools.Strengthen"
 -- and "Documentation.SBV.Examples.ProofTools.Sum" for examples.
-induct :: (Show res, Queriable IO st res)
+induct :: (Show res, Queriable IO st, res ~ QueryResult st)
        => Bool                             -- ^ Verbose mode
        -> Symbolic ()                      -- ^ Setup code, if necessary. (Typically used for 'Data.SBV.setOption' calls. Pass @return ()@ if not needed.)
        -> (st -> SBool)                    -- ^ Initial condition
@@ -92,7 +94,7 @@ induct :: (Show res, Queriable IO st res)
 induct = inductWith defaultSMTCfg
 
 -- | Induction engine, configurable with the solver
-inductWith :: (Show res, Queriable IO st res)
+inductWith :: (Show res, Queriable IO st, res ~ QueryResult st)
            => SMTConfig
            -> Bool
            -> Symbolic ()
@@ -139,7 +141,7 @@ inductWith cfg chatty setup initial trans strengthenings inv goal =
                                                   return $ Just ex
 
         strengthen []             cont = cont
-        strengthen ((nm, st):sts) cont = try ("Proving strengthening initation  : " ++ nm)
+        strengthen ((nm, st):sts) cont = try ("Proving strengthening initiation  : " ++ nm)
                                              (\s -> initial s .=> st s)
                                              (Failed (Initiation (Just nm)))
                                              $ try ("Proving strengthening consecution: " ++ nm)
