@@ -17,6 +17,8 @@
 {-# LANGUAGE MultiParamTypeClasses  #-}
 {-# LANGUAGE NamedFieldPuns         #-}
 {-# LANGUAGE ScopedTypeVariables    #-}
+{-# LANGUAGE TypeFamilies           #-}
+{-# LANGUAGE TypeOperators          #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -206,7 +208,7 @@ instance Show res => Show (ProofResult res) where
 
 
 -- | Checking WP based correctness
-wpProveWith :: forall st res. (Show res, Mergeable st, Queriable IO st res) => WPConfig -> Program st -> IO (ProofResult res)
+wpProveWith :: forall st res. (Show res, Mergeable st, Queriable IO st, res ~ QueryResult st) => WPConfig -> Program st -> IO (ProofResult res)
 wpProveWith cfg@WPConfig{wpVerbose} Program{setup, precondition, program, postcondition, stability} =
    runSMTWith (wpSolver cfg) $ do setup
                                   query q
@@ -339,7 +341,7 @@ wpProveWith cfg@WPConfig{wpVerbose} Program{setup, precondition, program, postco
                                 ++ measureDecreases   st'
 
 -- | Check correctness using the default solver. Equivalent to @'wpProveWith' 'defaultWPCfg'@.
-wpProve :: (Show res, Mergeable st, Queriable IO st res) => Program st -> IO (ProofResult res)
+wpProve :: (Show res, Mergeable st, Queriable IO st, res ~ QueryResult st) => Program st -> IO (ProofResult res)
 wpProve = wpProveWith defaultWPCfg
 
 -- | Configuration for WP proofs.
@@ -460,7 +462,7 @@ traceExecution Program{precondition, program, postcondition, stability} start =
                    | currentInvariant st
                    = while 1 st Nothing (Good st)
                    | True
-                   = stop loc  (InvariantPre loopName st) $ tag "invariant fails to hold prior to loop entry"
+                   = stop loc (InvariantPre loopName st) $ tag "invariant fails to hold prior to loop entry"
                    where tag s = "Loop " ++ show loopName ++ ": " ++ s
 
                          hasMeasure = isJust mbMeasure
@@ -486,4 +488,4 @@ traceExecution Program{precondition, program, postcondition, stability} start =
                            where mCur = currentMeasure is
                                  zero = map (const 0) mCur
 
-{-# ANN traceExecution ("HLint: ignore Use fromMaybe" :: String) #-}
+{- HLint ignore traceExecution "Use fromMaybe" -}
