@@ -26,6 +26,8 @@ module Data.SBV.Tuple (
     (^.), _1, _2, _3, _4, _5, _6, _7, _8
   -- * Tupling and untupling
   , tuple, untuple
+  -- * Swapping, only for 2-tuples
+  , swap
   ) where
 
 import GHC.TypeLits
@@ -34,12 +36,10 @@ import Data.SBV.Core.Data
 import Data.SBV.Core.Symbolic
 import Data.SBV.Core.Model
 
--- For doctest use only
---
 -- $setup
+-- >>> -- For doctest purposes only:
 -- >>> :set -XTypeApplications
--- >>> import Data.SBV.Provers.Prover (prove)
--- >>> import Data.SBV.Core.Model
+-- >>> import Data.SBV
 
 -- | Field access, inspired by the lens library. This is merely reverse
 -- application, but allows us to write things like @(1, 2)^._1@ which is
@@ -50,6 +50,11 @@ import Data.SBV.Core.Model
 t ^. f = f t
 infixl 8 ^.
 
+-- | Swap the elements of a 2-tuple
+swap :: (SymVal a, SymVal b) => STuple a b -> STuple b a
+swap t = tuple (b, a)
+  where (a, b) = untuple t
+
 -- | Dynamic interface to exporting tuples, this function is not
 -- exported on purpose; use it only via the field functions '_1', '_2', etc.
 symbolicFieldAccess :: (SymVal a, HasKind tup) => Int -> SBV tup -> SBV a
@@ -428,4 +433,4 @@ instance ( SymVal a, Metric a
                         msMaximize (nm ++ "^._7") (p^._7)
                         msMaximize (nm ++ "^._8") (p^._8)
 
-{-# ANN module ("HLint: ignore Reduce duplication" :: String) #-}
+{- HLint ignore module "Reduce duplication" -}
