@@ -13,18 +13,18 @@
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
-module Data.SBV.Utils.SExpr (SExpr(..), parenDeficit, parseSExpr, parseSExprFunction) where
+module Data.SBV.Utils.SExpr (SExpr(..), parenDeficit, parseSExpr, parseSExprFunction, makeHaskellFunction) where
 
 import Data.Bits   (setBit, testBit)
 import Data.Char   (isDigit, ord, isSpace)
 import Data.Either (partitionEithers)
-import Data.List   (isPrefixOf, nubBy)
+import Data.List   (isPrefixOf, nubBy, intercalate)
 import Data.Maybe  (fromMaybe, listToMaybe)
 import Data.Word   (Word32, Word64)
 
 import Control.Monad (foldM)
 
-import Numeric    (readInt, readDec, readHex, fromRat)
+import Numeric    (readInt, readSigned, readDec, readHex, fromRat)
 
 import Data.SBV.Core.AlgReals
 import Data.SBV.Core.SizedFloats
@@ -126,7 +126,7 @@ parseSExpr inp = do (sexp, extras) <- parse inpToks
         pTok ('#':'b':r)                                 = mkNum (Just (length r))     $ readInt 2 (`elem` "01") (\c -> ord c - ord '0') r
         pTok ('#':'x':r)                                 = mkNum (Just (4 * length r)) $ readHex r
 
-        pTok n | possiblyNum n = if all intChar n then mkNum Nothing $ readDec n else getReal n
+        pTok n | possiblyNum n = if all intChar n then mkNum Nothing $ readSigned readDec n else getReal n
         pTok n                 = return $ ECon (constantMap n)
 
         -- crude, but effective!
@@ -175,6 +175,23 @@ parseSExpr inp = do (sexp, extras) <- parse inpToks
                                                                    _               -> return $ EReal aval
                                                   _           -> die $ "Cannot parse a CVC4 approximate value from: " ++ show x
 
+        -- Deal with CVC5's algebraic reals. This is very crude!
+        cvt x@(EApp (ECon "_" : ECon "real_algebraic_number" : rest)) =
+            let isComma (ECon ",") = True
+                isComma _          = False
+
+                get (ENum    (n, _))               = return $ fromIntegral n
+                get (EReal   (AlgRational True r)) = return r
+                get (EFloat  f)                    = return $ toRational f
+                get (EDouble d)                    = return $ toRational d
+                get t                              = die $ "Cannot get a CVC5 real-algebraic bound from: " ++ show t
+
+            in case drop 1 (dropWhile (not . isComma) rest) of
+                [EApp [n1, n2], _] -> do low  <- get n1
+                                         high <- get n2
+                                         return $ EReal $ AlgInterval (OpenPoint low) (OpenPoint high)
+                _                  -> die $ "Cannot parse a CVC5 real-algebraic number from: " ++ show x
+
         -- NB. Note the lengths on the mantissa for the following two are 23/52; not 24/53!
         cvt (EApp [ECon "fp",    ENum (s, Just 1), ENum ( e, Just 8),  ENum (m, Just 23)])           = return $ EFloat         $ getTripleFloat  s e m
         cvt (EApp [ECon "fp",    ENum (s, Just 1), ENum ( e, Just 11), ENum (m, Just 52)])           = return $ EDouble        $ getTripleDouble s e m
@@ -366,10 +383,12 @@ parseLambdaExpression :: SExpr -> Maybe ([([SExpr], SExpr)], SExpr)
 parseLambdaExpression funExpr = case funExpr of
                                   EApp [ECon "lambda", EApp params, body] -> mapM getParam params >>= flip lambda body >>= chainAssigns
                                   _                                       -> Nothing
-  where getParam (EApp [ECon v, _]) = Just v
-        getParam _                  = Nothing
+  where getParam (EApp [ECon v, ECon ty]) = Just (v, ty == "Bool")
+        getParam (EApp [ECon v, _      ]) = Just (v, False)
+        getParam _                        = Nothing
 
-        lambda :: [String] -> SExpr -> Maybe [Either ([SExpr], SExpr) SExpr]
+        lambda :: [(String, Bool)]  -- Bool is True if this is a boolean variable. Otherwise we don't keep track of the type
+               -> SExpr -> Maybe [Either ([SExpr], SExpr) SExpr]
         lambda params body = reverse <$> go [] body
           where true  = ENum (1, Nothing)
                 false = ENum (0, Nothing)
@@ -422,23 +441,27 @@ parseLambdaExpression funExpr = case funExpr of
                 -- select takes the condition of an ite, and returns precisely what match is done to the parameters
                 select :: SExpr -> Maybe [SExpr]
                 select e
-                   | Just dict <- build e [] = mapM (`lookup` dict) params
+                   | Just dict <- build e [] = mapM (`lookup` dict) paramNames
                    | True                    = Nothing
-                  where -- build a dictionary of assignments from the scrutinee
+                  where paramNames = map fst params
+
+                        -- build a dictionary of assignments from the scrutinee
                         build :: SExpr -> [(String, SExpr)] -> Maybe [(String, SExpr)]
                         build (EApp (ECon "and" : rest)) sofar = let next _ Nothing  = Nothing
                                                                      next c (Just x) = build c x
                                                                  in foldr next (Just sofar) rest
 
-                        build expr sofar | Just (v, r) <- grok expr, v `elem` params = Just $ (v, r) : sofar
-                                         | True                                      = Nothing
+                        build expr sofar | Just (v, r) <- grok expr, v `elem` paramNames = Just $ (v, r) : sofar
+                                         | True                                          = Nothing
 
                         -- See if we can figure out what z3 is telling us; hopefully this
                         -- mapping covers everything we can see:
                         grok (EApp [ECon "=", ECon v, r]) = Just (v, r)
                         grok (EApp [ECon "=", r, ECon v]) = Just (v, r)
                         grok (EApp [ECon "not", ECon v])  = Just (v, false) -- boolean negation, require it to be false
-                        grok (ECon v)                     = Just (v, true)  -- boolean identity, require it to be true
+                        grok (ECon v)                     = case v `lookup` params of
+                                                               Just True -> Just (v, true)  -- boolean identity, require it to be true
+                                                               _         -> Nothing
 
                         -- Tough luck, we couldn't understand:
                         grok _ = Nothing
@@ -514,4 +537,88 @@ chainAssigns chain = regroup $ partitionEithers chain
         eRank EDouble{}        = 5
         eRank EApp{}           = 6
 
-{-# ANN chainAssigns ("HLint: ignore Redundant if" :: String) #-}
+-- Turn
+--  "((F (lambda ((x!1 Int)) (+ 3 (* 2 x!1)))))"
+---  into
+--  "F x = 3 + 2 * x"
+-- if we can. We try but don't push too hard! This is only used for display purposes.
+--
+-- This isn't very fool-proof; can be confused if there are binding constructs etc.
+-- Also, the generated text isn't necessarily fully Haskell acceptable.
+-- But it seems to do an OK job for most common use cases.
+makeHaskellFunction :: String -> String -> Maybe [String] -> Maybe String
+makeHaskellFunction resp nm mbArgs
+   = case parseSExpr resp of
+       Right (EApp [EApp [ECon o, e]]) | o == nm -> do (args, bd) <- lambda e
+                                                       return $ unwords (nm : args) ++ " = " ++ bd
+       _                                         -> Nothing
+
+  where -- infinite supply of names; starting with the ones we're given
+        preSupply = fromMaybe [] mbArgs
+
+        extras =  ["x", "y", "z"]
+               ++ [[c] | c <- ['a' .. 'z'], c < 'x']
+               ++ ['x' : show i | i <- [(1::Int) ..]]
+
+        mkUnique x | x `elem` preSupply = mkUnique $ x ++ "'"
+                   | True               = x
+
+        supply = preSupply ++ map mkUnique extras
+
+        lambda :: SExpr -> Maybe ([String], String)
+        lambda (EApp [ECon "lambda", EApp args, bd]) = do as <- mapM getArg args
+                                                          let env = zip as supply
+                                                          pure (map snd env, body env bd)
+        lambda _                                     = Nothing
+
+        getArg (EApp [ECon argName, _]) = Just argName
+        getArg _                        = Nothing
+
+        body env = go
+          where go :: SExpr -> String
+                go (ECon n) | Just a <- lookup n env = a
+                            | True                   = n
+
+                go (ENum (i, _))      = show i
+                go (EReal  a)         = show a
+                go (EFloat f)         = show f
+                go (EFloatingPoint f) = show f
+                go (EDouble f)        = show f
+                go (EApp xs)          = app (map (wrap . go) xs)
+
+                parens x = '(' : x ++ ")"
+
+                wrap x | "-" `isPrefixOf` x = parens x
+                       | any isSpace x      = parens x
+                       | True               = x
+
+                mkBin o a b = wrap a ++ " " ++ o ++ " " ++ wrap b
+
+                isPlus  = (`elem` ["+",  "bvadd"])
+                isTimes = (`elem` ["*",  "bvmul"])
+                isLT    = (`elem` ["<",  "bvult", "bvslt", "fp.lt" ])
+                isLTE   = (`elem` ["<=", "bvule", "bvsle", "fp.leq"])
+                isGT    = (`elem` [">",  "bvugt", "bvsgt", "fp.gt" ])
+                isGTE   = (`elem` [">=", "bvuge", "bvsge", "fp.gte"])
+
+                -- Make an application, with some simplifications
+                app :: [String] -> String
+                app (o : xs) | isPlus o = intercalate " + " xs
+
+                -- multiplication of arbitrary elements, with proviso for multiplication by -1
+                app [m, "(-1)", x] | isTimes m = '-' : x
+                app (m  : xs)      | isTimes m = intercalate " * " xs
+
+                -- binary arith ops
+                app ["-",  a, b] = mkBin "-"  a b
+                app ["/",  a, b] = mkBin "/"  a b
+
+                app [o, a, b] | isLT  o = mkBin "<"  a b
+                app [o, a, b] | isLTE o = mkBin "<=" a b
+                app [o, a, b] | isGT  o = mkBin ">"  a b
+                app [o, a, b] | isGTE o = mkBin ">=" a b
+
+                -- give up, and just do prefix!
+                app xs = unwords xs
+
+{- HLint ignore chainAssigns "Redundant if" -}
