@@ -141,8 +141,8 @@ decrypt key ct = map cvt $ zipWith xor (keyScheduleString key) ct
 -- large symbolic trace.
 rc4IsCorrect :: IO ThmResult
 rc4IsCorrect = prove $ do
-        key <- mkForallVars 5
-        pt  <- mkForallVars 5
+        key <- mkFreeVars 5
+        pt  <- mkFreeVars 5
         let ks  = keySchedule key
             ct  = zipWith xor ks pt
             pt' = zipWith xor ks ct
