@@ -15,6 +15,10 @@ module Documentation.SBV.Examples.Optimization.LinearOpt where
 
 import Data.SBV
 
+-- $setup
+-- >>> -- For doctest purposes only:
+-- >>> import Data.SBV
+
 -- | Taken from <http://people.brunel.ac.uk/~mastjjb/jeb/or/morelp.html>
 --
 --    *  maximize 5x1 + 6x2
@@ -31,7 +35,7 @@ import Data.SBV
 --   x1   =  47 % 9 :: Real
 --   x2   =  20 % 9 :: Real
 --   goal = 355 % 9 :: Real
-problem :: Goal
+problem :: ConstraintSet
 problem = do [x1, x2] <- mapM sReal ["x1", "x2"]
 
              constrain $ x1 + x2 .<= 10
