@@ -15,7 +15,11 @@ module Documentation.SBV.Examples.Optimization.VM where
 
 import Data.SBV
 
--- | The allocation problem. Inspired by: <http://rise4fun.com/Z3/tutorialcontent/optimization#h25>
+-- $setup
+-- >>> -- For doctest purposes only:
+-- >>> import Data.SBV
+
+-- | Computer allocation problem:
 --
 --   - We have three virtual machines (VMs) which require 100, 50 and 15 GB hard disk respectively.
 --
@@ -44,7 +48,7 @@ import Data.SBV
 --   cost        =    20 :: Integer
 --
 -- That is, we should put all the jobs on the third server, for a total cost of 20.
-allocate :: Goal
+allocate :: ConstraintSet
 allocate = do
     -- xij means VM i is running on server j
     x1@[x11, x12, x13] <- sBools ["x11", "x12", "x13"]
