@@ -26,7 +26,8 @@ import Data.String
 import Data.SBV
 import Data.SBV.Control
 
-import Data.SBV.String ((.++))
+import Prelude hiding ((++))
+import Data.SBV.String ((++))
 import qualified Data.SBV.RegExp as R
 
 -- | Simple expression language
@@ -47,9 +48,9 @@ type M = StateT (SArray String String) (WriterT [SString] Symbolic)
 eval :: SQLExpr -> M SString
 eval (Query q)         = do q' <- eval q
                             tell [q']
-                            lift $ lift exists_
+                            lift $ lift free_
 eval (Const str)       = return $ literal str
-eval (Concat e1 e2)    = (.++) <$> eval e1 <*> eval e2
+eval (Concat e1 e2)    = (++) <$> eval e1 <*> eval e2
 eval (ReadVar nm)      = do n   <- eval nm
                             arr <- get
                             return $ readArray arr n
