@@ -12,7 +12,6 @@
 -----------------------------------------------------------------------------
 
 {-# LANGUAGE DeriveAnyClass        #-}
-{-# LANGUAGE DeriveFoldable        #-}
 {-# LANGUAGE DeriveGeneric         #-}
 {-# LANGUAGE DeriveTraversable     #-}
 {-# LANGUAGE FlexibleInstances     #-}
@@ -30,6 +29,12 @@ import Data.SBV.Tools.WeakestPreconditions
 
 import GHC.Generics (Generic)
 
+-- $setup
+-- >>> -- For doctest purposes only:
+-- >>> import Data.SBV
+-- >>> import Data.SBV.Control
+-- >>> import Data.SBV.Tools.WeakestPreconditions
+
 -- * Program state
 
 -- | The state for the swap program, parameterized over a base type @a@.
@@ -42,9 +47,7 @@ data IncS a = IncS { x :: a    -- ^ Input value
 -- but we want it to be a little prettier here, and hence the @OVERLAPS@ directive.
 instance {-# OVERLAPS #-} (SymVal a, Show a) => Show (IncS (SBV a)) where
    show (IncS x y) = "{x = " ++ sh x ++ ", y = " ++ sh y ++ "}"
-     where sh v = case unliteral v of
-                    Nothing -> "<symbolic>"
-                    Just l  -> show l
+     where sh v = maybe "<symbolic>" show (unliteral v)
 
 -- | 'Fresh' instance for the program state
 instance SymVal a => Fresh IO (IncS (SBV a)) where
@@ -75,7 +78,7 @@ algorithm before after = Seq [ before
 pre :: I -> SBool
 pre IncS{x} = x .>= 0
 
--- | Postcondition for our program: @y@ must @x+1@.
+-- | Postcondition for our program: @y@ must equal @x+1@.
 post :: I -> SBool
 post IncS{x, y} = y .== x+1
 
