@@ -15,7 +15,6 @@
 -----------------------------------------------------------------------------
 
 {-# LANGUAGE DeriveAnyClass        #-}
-{-# LANGUAGE DeriveFoldable        #-}
 {-# LANGUAGE DeriveGeneric         #-}
 {-# LANGUAGE DeriveTraversable     #-}
 {-# LANGUAGE FlexibleInstances     #-}
@@ -37,6 +36,12 @@ import GHC.Generics (Generic)
 import Prelude hiding (gcd)
 import qualified Prelude as P (gcd)
 
+-- $setup
+-- >>> -- For doctest purposes only:
+-- >>> import Data.SBV
+-- >>> import Data.SBV.Control
+-- >>> import Data.SBV.Tools.WeakestPreconditions
+
 -- * Program state
 
 -- | The state for the sum program, parameterized over a base type @a@.
@@ -51,9 +56,7 @@ data GCDS a = GCDS { x :: a    -- ^ First value
 -- but we want it to be a little prettier here, and hence the @OVERLAPS@ directive.
 instance {-# OVERLAPS #-} (SymVal a, Show a) => Show (GCDS (SBV a)) where
    show (GCDS x y i j) = "{x = " ++ sh x ++ ", y = " ++ sh y ++ ", i = " ++ sh i ++ ", j = " ++ sh j ++ "}"
-     where sh v = case unliteral v of
-                    Nothing -> "<symbolic>"
-                    Just l  -> show l
+     where sh v = maybe "<symbolic>" show (unliteral v)
 
 -- | 'Fresh' instance for the program state
 instance SymVal a => Fresh IO (GCDS (SBV a)) where
@@ -118,28 +121,9 @@ gcd x y
 -- | Constraints and axioms we need to state explicitly to tell
 -- the SMT solver about our specification for GCD.
 axiomatizeGCD :: Symbolic ()
-axiomatizeGCD = do -- Base case. Strictly speaking, we don't really need this case
-                   -- here, but it declares the presence of gcd as an uninterpreted
-                   -- function to SBV so it gets registered as such.
-                   x <- sInteger_
-                   constrain $ gcd x x .== x
-
-                   -- Unfortunately; SBV does not support adding quantified constraints
-                   -- in the query mode. So we have to write this axiom directly in SMT-Lib.
-                   -- Note also how carefully we've chosen these axioms to work with our proof!
-                   -- Actually proving these is beyond the scope of our WP proof, but obviously
-                   -- should be done in some other system. (Note that SMT solvers will have hard
-                   -- time with the definition of GCD in general as the axiomatization requires
-                   -- quantification and definition requires recursion.)
-                   addAxiom "gcd_equal"    [ "(assert (forall ((x Int))"
-                                           , "                (=> (> x 0) (= (gcd x x) x))))"
-                                           ]
-                   addAxiom "gcd_unequal1" [ "(assert (forall ((x Int) (y Int))"
-                                           , "                (=> (and (> x 0) (> y 0)) (= (gcd (+ x y) y) (gcd x y)))))"
-                                           ]
-                   addAxiom "gcd_unequal2" [ "(assert (forall ((x Int) (y Int))"
-                                           , "                (=> (and (> x 0) (> y 0)) (= (gcd x (+ y x)) (gcd x y)))))"
-                                           ]
+axiomatizeGCD = do constrain $ \(Forall x)            -> x .> 0            .=> gcd x x     .== x
+                   constrain $ \(Forall x) (Forall y) -> x .> 0 .&& y .> 0 .=> gcd (x+y) y .== gcd x y
+                   constrain $ \(Forall x) (Forall y) -> x .> 0 .&& y .> 0 .=> gcd x (y+x) .== gcd x y
 
 -- | Precondition for our program: @x@ and @y@ must be strictly positive
 pre :: G -> SBool
