@@ -31,9 +31,7 @@ benchmarks = rGroup
              ]
   where _elts = \(x::SE) -> x .== x
         _four = \a b c (d::SE) -> distinct [a, b, c, d]
-        _maxE = do mx <- exists "maxE"
-                   e  <- forall "e"
-                   return $ mx .>= (e::SE)
-        _minE = do mx <- exists "minE"
-                   e  <- forall "e"
-                   return $ mx .<= (e::SE)
+        _maxE = do mx <- free "maxE"
+                   constrain $ \(Forall e) -> mx .>= (e::SE)
+        _minE = do mx <- free "minE"
+                   constrain $ \(Forall e) -> mx .<= (e::SE)
