@@ -23,5 +23,4 @@ import BenchSuite.Bench.Bench as S
 -- benchmark suite
 benchmarks :: Runner
 benchmarks = rGroup [ S.run "Counts" countPgm ]
- where countPgm = puzzle `fmap` mkExistVars 10
-       -- puzzle' d0 d1 d2 d3 d4 d5 d6 d7 d8 d9 = puzzle [d0, d1, d2, d3, d4, d5, d6, d7, d8, d9]
+ where countPgm = puzzle `fmap` mkFreeVars 10
