@@ -34,4 +34,4 @@ benchmarks = rGroup
   ]
 
 mkQueens :: Int -> Symbolic SBool
-mkQueens n = isValid n `fmap` mkExistVars n
+mkQueens n = isValid n `fmap` mkFreeVars n
