@@ -25,9 +25,9 @@ benchmarks :: Runner
 benchmarks = rGroup
   [ run "test" t `using` runner proveWith
   ]
-  where t = do addAxiom "OR distributes over AND" ax1
-               addAxiom "de Morgan"               ax2
-               addAxiom "double negation"         ax3
+  where t = do constrain $ \(Forall p) (Forall q) (Forall r) -> (p `or` q) `and` (p `or` r) .== p `or` (q `and` r)
+               constrain $ \(Forall p) (Forall q)            -> not (p `or` q) .== not p `and` not q
+               constrain $ \(Forall p)                       -> not (not p) .== p
                p <- free "p"
                q <- free "q"
                r <- free "r"
