@@ -31,10 +31,10 @@ mkTest tst i =
 -- prove that the "select" primitive is working correctly
 test1 :: Int -> IO Bool
 test1 n = isTheorem $ do
-            elts <- mkForallVars n
-            err  <- forall_
-            ind  <- forall_
-            ind2 <- forall_
+            elts <- mkFreeVars n
+            err  <- free_
+            ind  <- free_
+            ind2 <- free_
             let r1 = (select :: [SWord8] -> SWord8 -> SInt8 -> SWord8) elts err ind
                 r2 = (select :: [SWord8] -> SWord8 -> SWord8 -> SWord8) elts err ind2
                 r3 = slowSearch elts err ind
@@ -46,14 +46,14 @@ test1 n = isTheorem $ do
 
 test2 :: Int -> IO Bool
 test2 n = isTheorem $ do
-            elts1 <- mkForallVars n
-            elts2 <- mkForallVars n
+            elts1 <- mkFreeVars n
+            elts2 <- mkFreeVars n
             let elts = zip elts1 elts2
-            err1  <- forall_
-            err2  <- forall_
+            err1  <- free_
+            err2  <- free_
             let err = (err1, err2)
-            ind  <- forall_
-            ind2 <- forall_
+            ind  <- free_
+            ind2 <- free_
             let r1 = (select :: [(SWord8, SWord8)] -> (SWord8, SWord8) -> SInt8 -> (SWord8, SWord8)) elts err ind
                 r2 = (select :: [(SWord8, SWord8)] -> (SWord8, SWord8) -> SWord8 -> (SWord8, SWord8)) elts err ind2
                 r3 = slowSearch elts err ind
@@ -65,11 +65,11 @@ test2 n = isTheorem $ do
 
 test3 :: Int -> IO Bool
 test3 n = isTheorem $ do
-            eltsI <- mkForallVars n
+            eltsI <- mkFreeVars n
             let elts = map Left eltsI
-            errI  <- forall_
+            errI  <- free_
             let err = Left errI
-            ind  <- forall_
+            ind  <- free_
             let r1 = (select :: [Either SWord8 SWord8] -> Either SWord8 SWord8 -> SInt8 -> Either SWord8 SWord8) elts err ind
                 r2 = slowSearch elts err ind
             return $ r1 .== r2
