@@ -30,10 +30,7 @@ tests =
     ]
 
 crcGoodE :: Symbolic SBool
-crcGoodE = do
-  x <- exists_
-  y <- exists_
-  return (crcGood 3 0 x y)
+crcGoodE = pure $ quantifiedBool $ \(Exists x) (Exists y) -> crcGood 3 0 x y
 
 extendData :: SWord 48 -> SWord 64
 extendData msg = msg # 0
@@ -62,5 +59,5 @@ crcGood hd divisor sent received =
 mkPoly :: SWord 16 -> SWord 64
 mkPoly d = 1 # d
 
-{-# ANN crc_48_16 ("HLint: ignore Use camelCase" :: String) #-}
-{-# ANN crcGoodE  ("HLint: ignore Use <$>"       :: String) #-}
+{- HLint ignore crc_48_16 "Use camelCase" -}
+{- HLint ignore crcGoodE  "Use <$>"       -}
