@@ -21,11 +21,9 @@ import Utils.SBVTestFramework
 tests :: TestTree
 tests = testGroup "Existentials.CRCPolynomial" [
     goldenVsStringShow "crcPolyExist" (runSAT pgm)
-  , testCase "crcPolyGood" (assertIsSat pgm)
+  , testCase           "crcPolyGood"  (assertIsSat pgm)
  ]
 
 pgm :: Predicate
-pgm = do p <- exists "poly"
-         s <- forall "sent"
-         r <- forall "received"
-         return $ sTestBit p 0 .&& crcGood 4 p s r
+pgm = do poly <- free "poly"
+         pure $ quantifiedBool $ \(Forall sent) (Forall received) -> sTestBit poly 0 .&& crcGood 4 poly sent received
