@@ -6,7 +6,7 @@
 -- Maintainer: erkokl@gmail.com
 -- Stability : experimental
 --
--- Test suite for optimization iwth quantifiers
+-- Test suite for optimization with quantifiers
 -----------------------------------------------------------------------------
 
 {-# OPTIONS_GHC -Wall -Werror #-}
@@ -25,7 +25,7 @@ import qualified Control.Exception as C
 tests :: TestTree
 tests =
   testGroup "Optimization.Reals"
-    [ goldenString       "optQuant1" $ optE q1
+    [ goldenVsStringShow "optQuant1" $ opt  q1
     , goldenVsStringShow "optQuant2" $ opt  q2
     , goldenVsStringShow "optQuant3" $ opt  q3
     , goldenVsStringShow "optQuant4" $ opt  q4
@@ -35,50 +35,44 @@ tests =
           optE q = (show <$> optimize Lexicographic q) `C.catch` (\(e::C.SomeException) -> return (pick (show e)))
           pick s = unlines [l | l <- lines s, "***" `isPrefixOf` l]
 
-q1 :: Goal
+q1 :: ConstraintSet
 q1 = do a <- sInteger "a"
         [b1, b2] <- sIntegers ["b1", "b2"]
-        x <- forall "x" :: Symbolic SInteger
-        constrain $ 2 * (a * x + b1) .== 2
-        constrain $ 4 * (a * x + b2) .== 4
+        constrain $ \(Forall x) ->     2 * (a * x + b1) .== 2
+                                   .&& 4 * (a * x + b2) .== 4
         constrain $ a .>= 0
-        minimize "goal" $ 2*x
+        minimize "goal" $ 2*a
 
-q2 :: Goal
+q2 :: ConstraintSet
 q2 = do a <- sInteger "a"
         [b1, b2] <- sIntegers ["b1", "b2"]
-        x <- forall "x" :: Symbolic SInteger
-        constrain $ 2 * (a * x + b1) .== 2
-        constrain $ 4 * (a * x + b2) .== 4
-        constrain $ a .>= 0
+        constrain $ \(Forall x) ->     2 * (a * x + b1) .== 2
+                                   .&& 4 * (a * x + b2) .== 4
+                                   .&& a .>= 0
         minimize "goal" a
 
-q3 :: Goal
+q3 :: ConstraintSet
 q3 = do a <- sInteger "a"
         [b1, b2] <- sIntegers ["b1", "b2"]
         minimize "goal" a
-        x <- forall "x" :: Symbolic SInteger
-        constrain $ 2 * (a * x + b1) .== 2
-        constrain $ 4 * (a * x + b2) .== 4
+        constrain $ \(Forall x) ->     2 * (a * x + b1) .== 2
+                                   .&& 4 * (a * x + b2) .== 4
         constrain $ a .>= 0
 
-q4 :: Goal
+q4 :: ConstraintSet
 q4 = do a <- sInteger "a"
         [b1, b2] <- sIntegers ["b1", "b2"]
         minimize "goal" $ 2*a
-        x <- forall "x" :: Symbolic SInteger
-        constrain $ 2 * (a * x + b1) .== 2
-        constrain $ 4 * (a * x + b2) .== 4
+        constrain $ \(Forall x) ->     2 * (a * x + b1) .== 2
+                                   .&& 4 * (a * x + b2) .== 4
         constrain $ a .>= 0
 
-q5 :: Goal
+q5 :: ConstraintSet
 q5 = do a <- sInteger "a"
-        x <- forall "x" :: Symbolic SInteger
-        y <- forall "y" :: Symbolic SInteger
-        b <- sInteger "b"
+        constrain $ \(Exists y) -> a .>= (y :: SInteger)
         constrain $ a .>= 0
+        b <- sInteger "b"
         constrain $ b .>= 0
-        constrain $ x+y .>= 0
         minimize "goal" $ a+b
 
-{-# ANN module ("HLint: ignore Reduce duplication" :: String) #-}
+{- HLint ignore module "Reduce duplication" -}
