@@ -132,21 +132,16 @@ svNeg0 v = z `svMinus` v
 exactlyWhen :: SBool -> SVal -> SBool
 exactlyWhen (SBV a) b = SBV $ (a `svAnd` b) `svOr` (svNot a `svAnd` svNot b)
 
--- Properly extend to a dynamic large vector
+-- Properly extend to a dynamic signed large vector. This works because we grow to 256 bits, which is high enough.
 toLarge :: HasKind a => SBV a -> SLarge
 toLarge v
   | extra < 0 = error $ "toLarge: Unexpected size: " ++ show (n, large)
-  | hasSign v = p `svJoin` dv
-  | True      = z `svJoin` dv
+  | hasSign v =            svSignExtend extra (unSBV v)
+  | True      = mkSigned $ svZeroExtend extra (unSBV v)
   where n     = intSizeOf v
         extra = large - n
 
-        dv    = unSBV v
-        mk    = svInteger (KBounded True extra)
-        z     = mk 0
-        o     = mk (-1)
-        pos   = (dv `svTestBit` (n-1)) `svEqual` svFalse
-        p     = svIte pos z o
+        mkSigned = svFromIntegral (KBounded True large)
 
 -- Multiplication checks are expensive. For these, we simply check that the SBV encodings and the z3 versions are equivalent
 mulChkO :: forall a. SymVal a => (SBV a -> SBV a -> (SBool, SBool)) -> (SBV a -> SBV a -> (SBool, SBool)) -> Predicate
@@ -211,7 +206,6 @@ underflow op cond = do x  <- free "x"
                            extResult :: SLarge
                            extResult = toLarge x `op` toLarge y
 
-
                        return $ underflowHappens `exactlyWhen` (extResult `svLessThan` toLarge (minBound :: SBV a))
 
 overflow :: forall a. (Integral a, Bounded a, SymVal a) => (SLarge -> SLarge -> SLarge) -> (SBV a -> SBV a -> (SBool, SBool)) -> Predicate
@@ -245,4 +239,4 @@ overflow1 op cond = do x  <- free "x"
 
                        return $ overflowHappens `exactlyWhen` (extResult `svGreaterThan` toLarge (maxBound :: SBV a))
 
-{-# ANN module ("HLint: ignore Reduce duplication" :: String) #-}
+{- HLint ignore module "Reduce duplication" -}
