@@ -25,4 +25,4 @@ tests = testGroup "Puzzles.Coins" [
  where coinsPgm = runSAT $ do cs <- mapM mkCoin [1..6]
                               mapM_ constrain [c s | s <- combinations cs, length s >= 2, c <- [c1, c2, c3, c4, c5, c6]]
                               constrain $ sAnd $ zipWith (.>=) cs (tail cs)
-                              output $ sum cs .== 115
+                              pure $ sum cs .== 115
