@@ -26,4 +26,4 @@ tests =
 
 
 mkMagic :: Int -> Symbolic SBool
-mkMagic n = (isMagic . chunk n) `fmap` mkExistVars (n*n)
+mkMagic n = (isMagic . chunk n) `fmap` mkFreeVars (n*n)
