@@ -15,16 +15,16 @@ module TestSuite.Puzzles.Sudoku(tests) where
 
 import Documentation.SBV.Examples.Puzzles.Sudoku
 
+import Data.Maybe (fromMaybe)
 import Utils.SBVTestFramework
 
 tests :: TestTree
 tests =
   testGroup "Puzzles.Sudoku"
     [ testCase ("sudoku " ++ show n) (assert (checkPuzzle s))
-       | (n, s) <-
-           zip
-             [(0::Int)..]
-             [puzzle0, puzzle1, puzzle2, puzzle3, puzzle4, puzzle5, puzzle6] ]
+       | (n, s) <- zip [(0::Int)..] [puzzle1, puzzle2, puzzle3, puzzle4, puzzle5, puzzle6] ]
 
 checkPuzzle :: Puzzle -> IO Bool
-checkPuzzle (i, f) = isSatisfiable $ (valid . f) `fmap` mkExistVars i
+checkPuzzle p = do final <- fillBoard p
+                   let vld = valid (map (map literal) final)
+                   pure $ fromMaybe False (unliteral vld)
