@@ -12,6 +12,8 @@
 {-# LANGUAGE FlexibleContexts    #-}
 {-# LANGUAGE RankNTypes          #-}
 {-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TypeFamilies        #-}
+{-# LANGUAGE TypeOperators       #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -22,9 +24,8 @@ module Utils.SBVTestFramework (
         , goldenString
         , goldenVsStringShow
         , goldenCapturedIO
-        , CIOS(..), TestEnvironment(..), getTestEnvironment
         , qc1, qc2
-        , pickTests
+        , shouldNotTypeCheck
         -- module exports to simplify life
         , module Test.Tasty
         , module Test.Tasty.HUnit
@@ -33,15 +34,16 @@ module Utils.SBVTestFramework (
 
 import qualified Control.Exception as C
 
+import Control.DeepSeq     (force, NFData)
+import Control.Exception   (evaluate, try, throwIO, TypeError(..))
 import Control.Monad.Trans (liftIO)
 
 import qualified Data.ByteString.Lazy.Char8 as LBC
 
 import System.Directory   (removeFile)
-import System.Environment (lookupEnv)
 
 import Test.Tasty            (testGroup, TestTree, TestName)
-import Test.Tasty.HUnit      ((@?), Assertion, testCase, AssertionPredicable)
+import Test.Tasty.HUnit      ((@?), Assertion, testCase, AssertionPredicable, assertFailure)
 
 import Test.Tasty.Golden     (goldenVsString, goldenVsFileDiff)
 
@@ -49,50 +51,15 @@ import qualified Test.Tasty.QuickCheck   as QC
 import qualified Test.QuickCheck.Monadic as QC
 
 import Test.Tasty.Runners hiding (Result)
-import System.Random (randomRIO)
 
 import Data.SBV
 import Data.SBV.Control
 
-import Data.Char  (isDigit)
-import Data.Maybe (fromMaybe, catMaybes)
-
 import System.FilePath ((</>), (<.>))
+import Data.List       (isInfixOf, isSuffixOf)
 
-import Data.SBV.Internals (runSymbolic, Result, SBVRunMode(..), IStage(..), SBV(..), SVal(..), showModel, SMTModel(..), QueryContext(..))
 
----------------------------------------------------------------------------------------
--- Test environment; continuous integration
-data CIOS = CILinux
-          | CIOSX
-          | CIWindows
-          deriving (Show, Eq)
-
-data TestEnvironment = TestEnvLocal
-                     | TestEnvCI CIOS
-                     | TestEnvUnknown
-                     deriving Show
-
-getTestEnvironment :: IO (TestEnvironment, Int)
-getTestEnvironment = do mbTestEnv  <- lookupEnv "SBV_TEST_ENVIRONMENT"
-                        mbTestPerc <- lookupEnv "SBV_HEAVYTEST_PERCENTAGE"
-
-                        env <- case mbTestEnv of
-                                 Just "local" -> return   TestEnvLocal
-                                 Just "linux" -> return $ TestEnvCI CILinux
-                                 Just "osx"   -> return $ TestEnvCI CIOSX
-                                 Just "win"   -> return $ TestEnvCI CIWindows
-                                 Just other   -> do putStrLn $ "Ignoring unexpected test env value: " ++ show other
-                                                    return TestEnvUnknown
-                                 Nothing      -> return TestEnvUnknown
-
-                        perc <- case mbTestPerc of
-                                 Just n | all isDigit n -> return (read n)
-                                 Just n                 -> do putStrLn $ "Ignoring unexpected test percentage value: " ++ show n
-                                                              return 100
-                                 Nothing                -> return 100
-
-                        return (env, perc)
+import Data.SBV.Internals (runSymbolic, Result, SBVRunMode(..), IStage(..), SBV(..), SVal(..), showModel, SMTModel(..), QueryContext(..), Outputtable)
 
 -- | Generic assertion. This is less safe than usual, but will do.
 assert :: AssertionPredicable t => t -> Assertion
@@ -123,7 +90,7 @@ goldenCapturedIO n res = goldenVsFileDiff n diff gf gfTmp (rm gfTmp >> res gfTmp
 -- call this function if you provided a max-model count
 -- that was hit, or the search was stopped because the
 -- solver said 'Unknown' at some point.
-numberOfModels :: Provable a => a -> IO Int
+numberOfModels :: Satisfiable a => a -> IO Int
 numberOfModels p = do AllSatResult { allSatMaxModelCountReached  = maxHit
                                    , allSatSolverReturnedUnknown = unk
                                    , allSatSolverReturnedDSat    = ds
@@ -137,8 +104,8 @@ numberOfModels p = do AllSatResult { allSatMaxModelCountReached  = maxHit
                         _              -> return l
 
 -- | Symbolically run a SAT instance using the default config
-runSAT :: Symbolic a -> IO Result
-runSAT cmp = snd <$> runSymbolic (SMTMode QueryInternal ISetup True defaultSMTCfg) cmp
+runSAT :: Outputtable a => Symbolic a -> IO Result
+runSAT cmp = snd <$> runSymbolic defaultSMTCfg (SMTMode QueryInternal ISetup True defaultSMTCfg) (cmp >>= output >> pure ())
 
 -- | Turn provable to an assertion, theorem case
 assertIsThm :: Provable a => a -> Assertion
@@ -149,29 +116,20 @@ assertIsntThm :: Provable a => a -> Assertion
 assertIsntThm t = assert (fmap not (isTheorem t))
 
 -- | Turn provable to an assertion, satisfiability case
-assertIsSat :: Provable a => a -> Assertion
+assertIsSat :: Satisfiable a => a -> Assertion
 assertIsSat p = assert (isSatisfiable p)
 
 -- | Turn provable to a negative assertion, satisfiability case
-assertIsntSat :: Provable a => a -> Assertion
+assertIsntSat :: Satisfiable a => a -> Assertion
 assertIsntSat p = assert (fmap not (isSatisfiable p))
 
 -- | Quick-check a unary function, creating one version for constant folding, and another for solver
 qc1 :: (Eq a, SymVal a, SymVal b, Show a, QC.Arbitrary a, Eq b) => String -> (a -> b) -> (SBV a -> SBV b) -> [TestTree]
 qc1 nm opC opS = [cf, sm]
-   where cf = QC.testProperty (nm ++ ".constantFold") $ do
-                        i <- free "i"
-
-                        let grab n = fromMaybe (error $ "qc1." ++ nm ++ ": Cannot extract value for: " ++ n) . unliteral
-
-                            v = grab "i" i
-
-                            expected = literal $ opC v
-                            result   = opS i
-
-                        case (unliteral expected, unliteral result) of
-                           (Just _, Just _) -> return $ expected .== result
-                           _                -> return sFalse
+   where cf = QC.testProperty (nm ++ ".constantFold") $ \i ->
+                         case unliteral (opS (literal i)) of
+                             Just r -> opC i == r
+                             _      -> False
 
          sm = QC.testProperty (nm ++ ".symbolic") $ QC.monadicIO $ do
                         ((i, expected), result) <- QC.run $ runSMT $ do v   <- liftIO $ QC.generate QC.arbitrary
@@ -212,21 +170,10 @@ qc1 nm opC opS = [cf, sm]
 -- | Quick-check a binary function, creating one version for constant folding, and another for solver
 qc2 :: (Eq a, Eq b, SymVal a, SymVal b, SymVal c, Show a, Show b, QC.Arbitrary a, QC.Arbitrary b, Eq c) => String -> (a -> b -> c) -> (SBV a -> SBV b -> SBV c) -> [TestTree]
 qc2 nm opC opS = [cf, sm]
-   where cf = QC.testProperty (nm ++ ".constantFold") $ do
-                        i1 <- free "i1"
-                        i2 <- free "i2"
-
-                        let grab n = fromMaybe (error $ "qc2." ++ nm ++ ": Cannot extract value for: " ++ n) . unliteral
-
-                            v1 = grab "i1" i1
-                            v2 = grab "i2" i2
-
-                            expected = literal $ opC v1 v2
-                            result   = opS i1 i2
-
-                        case (unliteral expected, unliteral result) of
-                           (Just _, Just _) -> return $ expected .== result
-                           _                -> return sFalse
+   where cf = QC.testProperty (nm ++ ".constantFold") $ \i j ->
+                         case unliteral (opS (literal i) (literal j)) of
+                             Just r -> opC i j == r
+                             _      -> False
 
          sm = QC.testProperty (nm ++ ".symbolic") $ QC.monadicIO $ do
                         ((i1, i2, expected), result) <- QC.run $ runSMT $ do v1  <- liftIO $ QC.generate QC.arbitrary
@@ -267,19 +214,23 @@ qc2 nm opC opS = [cf, sm]
                            Right a -> QC.assert $ expected == a
                            _       -> QC.assert False
 
--- | Picking a certain percent of tests.
-pickTests :: Int -> TestTree -> IO TestTree
-pickTests d origTests = fromMaybe noTestsSelected <$> walk origTests
-   where noTestsSelected = TestGroup "pickTests.NoTestsSelected" []
 
-         walk t@SingleTest{}    = do c <- randomRIO (0, 99)
-                                     if c < d
-                                        then return $ Just t
-                                        else return Nothing
-         walk (TestGroup tn ts) = do cs <- catMaybes <$> mapM walk ts
-                                     case cs of
-                                       [] -> return Nothing
-                                       _  -> return $ Just $ TestGroup tn cs
-         walk _                 = error "pickTests: Unexpected test group!"
+-- Adapted from https://github.com/CRogers/should-not-typecheck/blob/2929b8303634fcfe200e8e37b744171aed3a757b/src/Test/ShouldNotTypecheck.hs#L1
+shouldNotTypeCheck :: NFData a => (() ~ () => a) -> Assertion
+shouldNotTypeCheck a = do
+  result <- try (evaluate $ force a)
+  case result of
+    Right _ -> assertFailure "Expected to not compile but it did compile."
 
-{-# ANN module ("HLint: ignore Reduce duplication" :: String) #-}
+    Left e@(TypeError msg)
+      | "(deferred type error)" `isSuffixOf` msg
+      -> case () of
+           () |     "No instance for" `isInfixOf` msg
+                 && "NFData"          `isInfixOf` msg
+              -> assertFailure $ "Make sure the expression has an NFData instance! Full error:\n" ++ msg
+              | True
+              -> pure ()
+      | True
+      -> throwIO e
+
+{- HLint ignore module "Reduce duplication" -}
